/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
using System;
using System.Xml;

namespace WeatherNotify
{
    #region events /eventargs
    /// <summary>
    /// used to raise Exception
    /// contains exception
    /// </summary>
    public class EventArgs_Exception:EventArgs
    {
        public readonly Exception exception;
        public EventArgs_Exception(Exception e)
        {
            exception=e;
        }
    }

    /// <summary>
    /// used to raise data
    /// contains string data
    /// </summary>
    public class EventArgs_StringData:EventArgs
    {
        public readonly string data="";
        public EventArgs_StringData(string data)
        {
            this.data=data;
        }
    }

    public delegate void MetarDownload_Error_EventHandler(ClassMetarDownload sender, EventArgs_Exception e);
    public delegate void MetarDownload_Arrival_EventHandler(ClassMetarDownload sender, EventArgs_StringData e);
    #endregion

    public class ClassResponseMetar
    {
        public string Metar="";
        public string Date = "";
    }

    public class ClassMetarDownload
    {
        public event MetarDownload_Error_EventHandler event_MetarDownload_Error;
        public event MetarDownload_Arrival_EventHandler event_MetarDownload_Arrival;
        public bool Started
        {
            get{return !this.b_stopped;}
        }

        // weather.noaa.gov ends to provide service in aougust 2016, use www.aviationweather.gov now
        // private string host="weather.noaa.gov";//"127.0.0.1";//"weather.noaa.gov";//
        // private string location="/pub/data/observations/metar/stations/";
        private string host = "www.aviationweather.gov";
        private string location="/adds/dataserver_current/httpparam?dataSource=metars&requestType=retrieve&format=xml&hoursBeforeNow=6&mostRecent=true&stationString=";
        private ushort host_port=80;
        private string proxy_ip="127.0.0.1";
        private ushort proxy_port=80;
        private easy_socket.proxy.ClassProxy.PROXY_TYPE proxy_type=easy_socket.proxy.ClassProxy.PROXY_TYPE.NONE;
        private UInt32 retry_sec=300;// 5min
        private bool b_stopped=true;
        private System.DateTime last_update;
        private int thread_delay_in_sec=0;
        private System.Threading.AutoResetEvent hevt_thread_start;
        private System.Threading.ManualResetEvent hevt_stop_event;
        private System.Threading.ManualResetEvent hevt_socket_closed;
        private string icao="";
        private System.Timers.Timer timer_update;
        private easy_socket.proxy.ClassProxy proxy;
        private string rcv_buff="";
        public enum eparsing_type : byte
        {
            parsing_type_Text=0,
            parsing_type_Xml
        }
        private eparsing_type parsing_type = eparsing_type.parsing_type_Text;

        public ClassMetarDownload()
        {
            this.hevt_thread_start=new System.Threading.AutoResetEvent(false);
            this.hevt_stop_event=new System.Threading.ManualResetEvent(false);
            this.hevt_socket_closed=new System.Threading.ManualResetEvent(false);
            this.timer_update=new System.Timers.Timer();
            this.timer_update.Enabled=false;
            this.timer_update.Elapsed+=new System.Timers.ElapsedEventHandler(timer_update_Elapsed);
            this.proxy=new easy_socket.proxy.ClassProxy();
            this.proxy.event_ClassProxy_Closed_by_Remote_Side+=new easy_socket.proxy.ClassProxy_Closed_by_Remote_Side_EventHandler(proxy_event_ClassProxy_Closed_by_Remote_Side);
            this.proxy.event_ClassProxy_DataArrival+=new easy_socket.proxy.ClassProxy_DataArrival_EventHandler(proxy_event_ClassProxy_DataArrival);
            this.proxy.event_ClassProxy_Error+=new easy_socket.proxy.ClassProxy_Error_EventHandler(proxy_event_ClassProxy_Error);
        }

        #region start/stop
        public void start(string icao, UInt32 update_interval_in_sec, UInt32 retry_in_sec, string host, string remote_path_folder, eparsing_type parsing_type)
        {
            this.start(icao, update_interval_in_sec, retry_in_sec, host, 80, remote_path_folder, false, "", 0, easy_socket.proxy.ClassProxy.PROXY_TYPE.NONE, parsing_type);
        }
        public void start(string icao,
                          UInt32 update_interval_in_sec,
                          UInt32 retry_in_sec,
                          string host,ushort host_port,string remote_path_folder,
                          bool b_use_proxy,string proxy_ip,ushort proxy_port,
                          easy_socket.proxy.ClassProxy.PROXY_TYPE proxy_type,
                          eparsing_type parsing_type)
        {
            this.stop();
            // assume that event has been thrown
            this.hevt_stop_event.WaitOne(1000,true);
            // let time to old threads to stop
            System.Threading.Thread.Sleep(200);

            this.retry_sec=retry_in_sec;
            this.location=remote_path_folder;
            this.host=host;
            this.host_port=host_port;
            this.proxy_ip=proxy_ip;
            this.proxy_port=proxy_port;
            if (b_use_proxy)
                this.proxy_type=proxy_type;
            else
                this.proxy_type=easy_socket.proxy.ClassProxy.PROXY_TYPE.NONE;
            this.icao=icao;
            this.timer_update.Interval=update_interval_in_sec*1000;
            this.parsing_type = parsing_type;
            this.hevt_stop_event.Reset();
            this.b_stopped=false;
            // create thread for mystart
            System.Threading.ThreadStart ths=new System.Threading.ThreadStart(mystart);
            System.Threading.Thread th=new System.Threading.Thread(ths);
            th.Start();
        }
        private void mystart()
        {
            // check get metar for first time
            this.start_download_delay(0,false);
            this.timer_update.Enabled=true;
        }
        
        public void stop()
        {
            // stop timer
            this.timer_update.Enabled=false;
            this.b_stopped=true;
            // set the hevt_stop_event
            this.hevt_stop_event.Set();
        }
        private void start_download_delay(UInt32 delay_in_sec,bool b_blocking)
        {
            if (this.b_download_already_delay)// a download is already delay
                return;
            this.b_download_already_delay=true;
            this.thread_delay_in_sec=(int)delay_in_sec;
            if (b_blocking)
            {
                this.start_download_delay();
            }
            else
            {
                System.Threading.ThreadStart ths=new System.Threading.ThreadStart(start_download_delay);
                System.Threading.Thread th=new System.Threading.Thread(ths);
                th.Start();
                this.hevt_thread_start.Reset();
                this.hevt_thread_start.WaitOne(3000,true);
            }
        }
        private bool b_download_already_delay=false;

        private void start_download_delay()
        {
            int delay_in_sec=this.thread_delay_in_sec;
            // data are saved continue
            this.hevt_thread_start.Set();

            if(this.hevt_stop_event.WaitOne(delay_in_sec*1000,true))// stop event occurs
            {
                this.b_download_already_delay=false;
                return;
            }
            // else

            // download
            bool b_success=this.get_metar_from_web();
            // a new download can be delay
            this.b_download_already_delay=false;
            // if error during download
            if ((!b_success)&&(!this.b_stopped))
                // start another delay download
                this.start_download_delay(this.retry_sec,false);
        }

        #endregion

        #region download
        /// <summary>
        /// The METAR is fetched via HTTP from the National Weather Services
        /// public server. The files can be found under the
        /// http://weather.noaa.gov/pub/data/observations/metar/stations/
        /// directory as ICAO.TXT where ICAO is replaced by the actual ICAO.
        /// weather.noaa.gov ends to provide service in august 2016, use www.aviationweather.gov now
        /// </summary>
        /// <param name="icao"></param>
        /// <returns>success status</returns>
        public bool get_metar_from_web()
        {
            string metar_data;
            string metar = "";
            string[] pdate;
            metar_data = this.get_metar_socket(this.icao);
            /* Here we test to see if we actually got a METAR. */
            if (metar_data!="")
            {
            /* http answer parsing before https / WebClient use
                string str_header="";
                int pos_end_of_header=metar_data.IndexOf("\r\n\r\n");
                if (pos_end_of_header>0)
                    str_header=metar_data.Substring(0,pos_end_of_header);
                if (!System.Text.RegularExpressions.Regex.IsMatch(str_header,"HTTP/[0-9]+.[0-9]+ 2[0-9]{2}",System.Text.RegularExpressions.RegexOptions.IgnoreCase))
                {
                    int pos_404_not_found=metar_data.IndexOf(" 404 ");
                    if ((pos_404_not_found>0)&&(pos_end_of_header>0)&&(pos_end_of_header>pos_404_not_found))
                    {
                        // pretty error message
                        // throw error
                        if (this.event_MetarDownload_Error!=null)
                            this.event_MetarDownload_Error(this,new EventArgs_Exception(new Exception("City informations not found on server "+this.host+" !")));
                    }
                    else // let message
                    {
                        // throw error
                        if (this.event_MetarDownload_Error!=null)
                            this.event_MetarDownload_Error(this,new EventArgs_Exception(new Exception("Bad data:\r\n"+metar_data)));
                    }
                    return false;
                }
                metar_data=metar_data.Substring(pos_end_of_header+4);
             */
                switch (this.parsing_type)
                {
                    case eparsing_type.parsing_type_Text:
                    {
                        // weather.noaa.gov way
                 
                        /* The first line in the file is the date */
                        int eol=metar_data.IndexOfAny(new char[]{'\r','\n'});
                        if (eol<0)
                        {
                            // throw error
                            if (this.event_MetarDownload_Error!=null)
                                this.event_MetarDownload_Error(this,new EventArgs_Exception(new Exception("Bad data:\r\n"+metar_data)));
                            return false;
                        }
                        string str_date = metar_data.Substring(0,eol).Trim();
                        // remove date from metar
                        metar_data = metar_data.Substring(eol+1);
                        /* The remaining lines are the METAR itself. This will merge the
                        * remaining lines into one line by removing new-lines:
                        */
                        // ereg_replace("[\n\r ]+", " ", );
                        metar = metar_data.Replace("\n"," ").Replace("\r","");
                        // first line contains date and is like is like 2004/11/28 10:00
                        pdate = str_date.Split(new char[]{':','/',' '});
                        // pdate is like is like [yyyy,mm,dd,hh,mm]
                        if (pdate.Length>=5)
                        {
                            this.last_update=new System.DateTime(System.Convert.ToInt32(pdate[0]),
                                                                System.Convert.ToInt32(pdate[1]),
                                                                System.Convert.ToInt32(pdate[2]),
                                                                System.Convert.ToInt32(pdate[3]),
                                                                System.Convert.ToInt32(pdate[4]),
                                                                0);
                        }
                        if (!ClassEreg.ereg("[0-9]{6}Z", metar))
                        {
                            /* Some reports don't even have a time-part, so we insert the
                            * current time. This might not be the time of the report, but
                            * it was broken anyway :-)
                            */
                            this.last_update=System.DateTime.Now;
                        }
                    }
                    break;

                    case eparsing_type.parsing_type_Xml:
                    {
                        int XmlStart = metar_data.IndexOf("<?xml");
                        // int XmlStart = metar_data.IndexOf("<response");
                        int XmlEnd = metar_data.IndexOf("</response>");
                        if ( (XmlStart < 0) || (XmlEnd<0) )
                        {
                            // throw error
                            if (this.event_MetarDownload_Error != null)
                                this.event_MetarDownload_Error(this, new EventArgs_Exception(new Exception("Bad data:\r\n" + metar_data)));
                            return false;
                        }

                        metar_data = metar_data.Substring(XmlStart, XmlEnd - XmlStart + "</response>".Length);

                        XmlDocument xmlDoc = new XmlDocument(); // Create an XML document object
                        xmlDoc.LoadXml(metar_data);

                        // XmlNodeList Nodes = xmlDoc.DocumentElement.SelectNodes("/response/data/METAR");
                        XmlNodeList Nodes = xmlDoc.DocumentElement.SelectNodes("/response/data/METAR");

                        if (Nodes.Count == 0)
                        {
                            // throw error
                            if (this.event_MetarDownload_Error != null)
                                this.event_MetarDownload_Error(this, new EventArgs_Exception(new Exception("No data available for selected city")));
                            return false;
                        }

                        // List<ClassResponseMetar> Metars = new List<ClassResponseMetar>();
                        ClassResponseMetar Metar = new ClassResponseMetar();

                        foreach (XmlNode Node in Nodes)
                        {
                            // ClassResponseMetar Metar = new ClassResponseMetar();
                            Metar.Metar = Node.SelectSingleNode("raw_text").InnerText;
                            Metar.Date = Node.SelectSingleNode("observation_time").InnerText;

                            // Metars.Add(book);
                        }

                        pdate = Metar.Date.Split(new char[] { ':', '/', ' ' });
                        // pdate is like is like [yyyy,mm,dd,hh,mm]
                        if (pdate.Length >= 5)
                        {
                            this.last_update = new System.DateTime(System.Convert.ToInt32(pdate[0]),
                                                                System.Convert.ToInt32(pdate[1]),
                                                                System.Convert.ToInt32(pdate[2]),
                                                                System.Convert.ToInt32(pdate[3]),
                                                                System.Convert.ToInt32(pdate[4]),
                                                                0);
                        }
                        if (!ClassEreg.ereg("[0-9]{6}Z", metar))
                        {
                            /* Some reports don't even have a time-part, so we insert the
                            * current time. This might not be the time of the report, but
                            * it was broken anyway :-)
                            */
                            this.last_update = System.DateTime.Now;
                        }
                        metar = Metar.Metar;
                    }
                    break;
                default:
                    return false;
                }

                // throw event metar arrival (metar)
                if (this.event_MetarDownload_Arrival!=null)
                {
                    try
                    {
                        this.event_MetarDownload_Arrival(this,new EventArgs_StringData(metar));
                    }
                    catch(Exception e)
                    {
                        return false;
                    }
                }
                return true;
            }
            else
            {
                // error retrieving metar
                // throw event metar arrival (metar) anyway to update interface with no data fields
                if (this.event_MetarDownload_Arrival!=null)
                {
                    try
                    {
                        this.event_MetarDownload_Arrival(this,new EventArgs_StringData(metar));
                    }
                    catch(Exception e)
                    {
                        return false;
                    }
                }
            }
            return false;
        }

        public string get_metar_socket(string icao)
        {
            // old noa web site
            // string strlocation = this.location+icao+".TXT";
            // string strlocation = this.location+icao;
            string strlocation = this.location.Replace("%METAR_NAME%",icao);           
            string metar_data = "";

            /* before https and WebClient use : raw http
             
            string HttpHeader = "HTTP/1.1\r\n";
            //                            +"If-Modified-Since: Sat, 29 Oct 1994 09:00:00 GMT\r\n"
            //                            +"Pragma: no-cache\r\n"
            //                            +"Cache-Control: no-cache\r\n";

            string request = "GET https://" + this.host + ":" + this.host_port;
            if (strlocation.Substring(0, 1) != "/")
                request += "/";
            request += strlocation;
            request += " " + HttpHeader
                        + "Host: " + this.host + "\r\n"
                        + "Content-Type: text/html\r\n"
                        + "Connection: Close\r\n\r\n";
            
            

            this.hevt_socket_closed.Reset();
            this.rcv_buff="";
            this.proxy.type=this.proxy_type;
            this.proxy.proxy_ip=this.proxy_ip;
            this.proxy.proxy_port=this.proxy_port;
            this.proxy.send(this.host,this.host_port,request);
            // wait for multiple object event stop /connection closed,error,(cancel not implemented,timeout included with socket timeout)
            System.Threading.WaitHandle.WaitAny(new System.Threading.WaitHandle[]{this.hevt_stop_event,this.hevt_socket_closed});
            metar_data=this.rcv_buff;
            this.proxy.close();// close if not already done
            
            */

			// System.Net.ServicePointManager.ServerCertificateValidationCallback = delegate { return true; };
			System.Net.ServicePointManager.CertificatePolicy = new MyCertificatePolicy();

            // System.Net.ServicePointManager.SecurityProtocol = System.Net.SecurityProtocolType.Tls | System.Net.SecurityProtocolType.Ssl3;
            // from https://www.medo64.com/2020/05/using-tls-1-3-from-net-4-0-application/
            try
            { //try TLS 1.3
                System.Net.ServicePointManager.SecurityProtocol = (System.Net.SecurityProtocolType)12288
                                                     | (System.Net.SecurityProtocolType)3072
                                                     | (System.Net.SecurityProtocolType)768
                                                     | System.Net.SecurityProtocolType.Tls;
            }
            catch (NotSupportedException)
            {
                try
                { //try TLS 1.2
                    System.Net.ServicePointManager.SecurityProtocol = (System.Net.SecurityProtocolType)3072
                                                         | (System.Net.SecurityProtocolType)768
                                                         | System.Net.SecurityProtocolType.Tls;
                }
                catch (NotSupportedException)
                {
                    try
                    { //try TLS 1.1
                        System.Net.ServicePointManager.SecurityProtocol = (System.Net.SecurityProtocolType)768
                                                             | System.Net.SecurityProtocolType.Tls;
                    }
                    catch (NotSupportedException)
                    { //TLS 1.0
                        System.Net.ServicePointManager.SecurityProtocol = System.Net.SecurityProtocolType.Tls;
                    }
                }
            }


            System.Net.WebClient webClient = new System.Net.WebClient();
			// System.Net.WebClient.DownloadString not supported in .Net 1.1
            // metar_data = client.DownloadString("https://"+this.host + strlocation);
            try
            { 
				//webClient.Headers["User-Agent"] = "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.2.6) Gecko/20100625 Firefox/3.6.6 (.NET CLR 3.5.30729)";
				//webClient.Headers["Accept"] = "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8";
				//webClient.Headers["Accept-Language"] = "en-us,en;q=0.5";
				//webClient.Headers["Accept-Encoding"] = "gzip,deflate";
				//webClient.Headers["Accept-Charset"] = "ISO-8859-1,utf-8;q=0.7,*;q=0.7";
				string url = "https://" + this.host + strlocation;

				System.IO.StreamReader sr = new  System.IO.StreamReader(webClient.OpenRead(url));
				metar_data = sr.ReadToEnd();
            }
            catch (System.Exception e)
            {
       	
            }
            
            return metar_data;
        }

		public class MyCertificatePolicy:System.Net.ICertificatePolicy
		{
			public bool CheckValidationResult(
												System.Net.ServicePoint srvPoint,
												System.Security.Cryptography.X509Certificates.X509Certificate certificate,
												System.Net.WebRequest request,
												int certificateProblem
												)
			{
				return true;
			}
		}

        private void timer_update_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            this.start_download_delay(0,false);
        }

        private void proxy_event_ClassProxy_Closed_by_Remote_Side(easy_socket.proxy.ClassProxy sender, EventArgs e)
        {
            // event
            this.hevt_socket_closed.Set();
        }

        private void proxy_event_ClassProxy_DataArrival(easy_socket.proxy.ClassProxy sender, easy_socket.tcp.EventArgs_ReceiveDataSocket e)
        {
            this.rcv_buff+=System.Text.Encoding.Default.GetString(e.buffer , 0, e.buffer_size );
        }

        private void proxy_event_ClassProxy_Error(easy_socket.proxy.ClassProxy sender, easy_socket.tcp.EventArgs_Exception e)
        {
            // throw error
            if (this.event_MetarDownload_Error!=null)
                this.event_MetarDownload_Error(this,new EventArgs_Exception(e.exception));
            // event
            this.hevt_socket_closed.Set();
        }
        #endregion

    }
}
