/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace WeatherNotify
{
	public class FormMain : System.Windows.Forms.Form
	{

		[System.Runtime.InteropServices.DllImport("user32.dll", CharSet=System.Runtime.InteropServices.CharSet.Auto)]
		extern static bool DestroyIcon(IntPtr handle);

		private System.ComponentModel.IContainer components;
        
		private const string IMG_DIRECTORY="icons\\";
		private const string CONFIG_FILENAME="options.xml";
		private const string LANGUAGE_DIRECTORY="languages\\";
		private const string LANGUAGE_INTERFACE_DIRECTORY="languages\\interface\\";
		private const string CITY_FILENAME="stations.db";

		private ClassMetarDownload obj_metar_download=null;
		private ClassInterfaceLanguage obj_interface_language=null;
		private ClassOptions obj_options;

		private FormWeatherView frm_weather_view;
		private FormDetails frm_details=null;
		private FormAbout frm_about;
		private System.Windows.Forms.NotifyIcon[] array_NotifyIcon;
        
		private ClassCity[] pcity_list;
		private enum NOTIFY_ICON_POS{WEATHER=0,TEMP,WIND_IMG,WIND_DATA};
		private int notify_icon_number=4;
		private bool b_closed_forced=false;
		private string exe_path="";
		private string last_raw_metar="";
		private string[] report_language_available=null;
        


		private System.Windows.Forms.ContextMenu contextMenu_main;
		private System.Windows.Forms.MenuItem menuItem_exit;
		private System.Windows.Forms.MenuItem menuItem_about;
		private System.Windows.Forms.MenuItem menuItem_options;
		private System.Windows.Forms.MenuItem menuItem_hide_show;
		private System.Windows.Forms.Button button_apply;
		private System.Windows.Forms.Button button_cancel;
		private System.Windows.Forms.Button button_exit;
		private System.Windows.Forms.MenuItem menuItem_details;
		private System.Windows.Forms.GroupBox groupBox_language;
		private System.Windows.Forms.ComboBox comboBox_language;
		private System.Windows.Forms.GroupBox groupBox_current_state;
		private System.Windows.Forms.Label label_current_state;
		private System.Windows.Forms.Label label_city;
		private System.Windows.Forms.ComboBox comboBox_cities;
		private System.Windows.Forms.GroupBox groupBox_Window;
		private System.Windows.Forms.ComboBox comboBox_orientation;
		private System.Windows.Forms.Label label_orientation;
		private System.Windows.Forms.CheckBox checkBox_w_pressure;
		private System.Windows.Forms.Label label_window_opacity;
		private System.Windows.Forms.TrackBar trackBar_w_opacity;
		private System.Windows.Forms.CheckBox checkBox_w_topmost;
		private System.Windows.Forms.CheckBox checkBox_w_wind;
		private System.Windows.Forms.CheckBox checkBox_w_clouds;
		private System.Windows.Forms.CheckBox checkBox_w_temp;
		private System.Windows.Forms.GroupBox groupBox_notifyicon;
		private System.Windows.Forms.CheckBox checkBox_wind_speed;
		private System.Windows.Forms.CheckBox checkBox_wind_dir;
		private System.Windows.Forms.CheckBox checkBox_clouds;
		private System.Windows.Forms.CheckBox checkBox_temp;
		private System.Windows.Forms.TabPage tabPage_general;
		private System.Windows.Forms.TabPage tabPage_units;
		private System.Windows.Forms.TabPage tabPage_connection;
		private System.Windows.Forms.ComboBox comboBox_height_unit;
		private System.Windows.Forms.Label label_height_unit;
		private System.Windows.Forms.ComboBox comboBox_length_unit;
		private System.Windows.Forms.ComboBox comboBox_precipitation_height_unit;
		private System.Windows.Forms.Label label_length_unit;
		private System.Windows.Forms.Label label_precipitation_height_unit;
		private System.Windows.Forms.ComboBox comboBox_pressure_unit;
		private System.Windows.Forms.Label label_pressure_unit;
		private System.Windows.Forms.ComboBox comboBox_wind_unit;
		private System.Windows.Forms.ComboBox comboBox_temp_unit;
		private System.Windows.Forms.Label label_temp_unit;
		private System.Windows.Forms.Label label_wind_unit;
		private System.Windows.Forms.Label label_remote_host_ip;
		private System.Windows.Forms.TextBox textBox_remote_host;
		private System.Windows.Forms.Label label_remote_host_port;
		private System.Windows.Forms.TextBox textBox_remote_host_port;
		private System.Windows.Forms.Label label_remote_host_directory;
		private System.Windows.Forms.TextBox textBox_remote_host_directory;
		private System.Windows.Forms.CheckBox checkBox_use_proxy;
		private System.Windows.Forms.GroupBox groupBox_proxy;
		private System.Windows.Forms.Label label_proxy_port;
		private System.Windows.Forms.Label label_proxy_ip;
		private System.Windows.Forms.Label label_proxy_type;
		private System.Windows.Forms.ComboBox comboBox_proxy_type;
		private System.Windows.Forms.TabControl tabControl_main;
		private System.Windows.Forms.TextBox textBox_proxy_port;
		private System.Windows.Forms.TextBox textBox_proxy_ip;
		private System.Windows.Forms.TextBox textBox_update_interval;
		private System.Windows.Forms.Label label_update_interval;
		private System.Windows.Forms.TextBox textBox_retry_interval;
		private System.Windows.Forms.Label label_retry_interval;
		private ComboBox comboBox_reply_type;
		private CheckBox checkBox_notify_font_bold;
		private NumericUpDown numeric_UpDown_notify_font_size;
		private ComboBox comboBox_notify_font_family;
		private System.Windows.Forms.GroupBox groupBoxnotify_font;
		private System.Windows.Forms.Label label_notify_font_size;
		private System.Windows.Forms.Label label_notify_font_family;
		private System.Windows.Forms.Label label_received_data_type;
		private System.Windows.Forms.MenuItem menuItem_separator;


		#region constructor

		public FormMain()
		{
			InitializeComponent();
			XPStyle.MakeXPStyle(this);

			// get exe path
			this.exe_path=System.Windows.Forms.Application.ExecutablePath;
			int last_pos=this.exe_path.LastIndexOf("\\");
			if (last_pos>0)
				this.exe_path=this.exe_path.Substring(0,last_pos+1);

			// metar download
			this.obj_metar_download=new ClassMetarDownload();
			this.obj_metar_download.event_MetarDownload_Arrival+=new MetarDownload_Arrival_EventHandler(obj_metar_download_event_MetarDownload_Arrival);
			this.obj_metar_download.event_MetarDownload_Error+=new MetarDownload_Error_EventHandler(obj_metar_download_event_MetarDownload_Error);
            
			this.obj_options=ClassOptions.load(this.exe_path+CONFIG_FILENAME); 
			this.frm_details=new FormDetails();
			this.frm_weather_view=new FormWeatherView(this,ref this.obj_options,this.frm_details);
			this.frm_about=new FormAbout();
			this.update_language_interface();
            
			int cpt;

			// notify icons
			if (this.components==null)
				this.components = new System.ComponentModel.Container();
			this.array_NotifyIcon=new System.Windows.Forms.NotifyIcon[notify_icon_number];
			for (cpt=0;cpt<notify_icon_number;cpt++)
			{
				this.array_NotifyIcon[cpt] = new System.Windows.Forms.NotifyIcon(this.components);
				this.array_NotifyIcon[cpt].ContextMenu=this.contextMenu_main;
				this.array_NotifyIcon[cpt].DoubleClick+=new EventHandler(NotifyIcon_DoubleClick);
				this.array_NotifyIcon[cpt].Icon=null;
			}
			this.array_NotifyIcon[(int)NOTIFY_ICON_POS.WEATHER].Text =this.obj_interface_language.weather;
			this.array_NotifyIcon[(int)NOTIFY_ICON_POS.TEMP].Text = this.obj_interface_language.temperature;
			this.array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_IMG].Text = this.obj_interface_language.wind_direction;
			this.array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_DATA].Text = this.obj_interface_language.wind_speed;

			// city
			this.pcity_list=ClassCity.load_from_config_file(this.exe_path+CITY_FILENAME);

			//report language
			try
			{
				this.report_language_available=System.IO.Directory.GetFiles(this.exe_path+LANGUAGE_DIRECTORY,"*.xml");
				if (this.report_language_available!=null)
				{
					for (cpt=0;cpt<this.report_language_available.Length;cpt++)
						this.report_language_available[cpt]=System.IO.Path.GetFileNameWithoutExtension(this.report_language_available[cpt]);
					this.comboBox_language.Items.AddRange(this.report_language_available);
				}
			}
			catch
			{
				this.report_language_available=null;
			}

			// update options
			if (this.pcity_list!=null)
			{
				if (this.pcity_list.Length!=0)
				{
					for (int cnt=0;cnt<this.pcity_list.Length;cnt++)
						this.comboBox_cities.Items.Add(this.pcity_list[cnt].country +" : "+ this.pcity_list[cnt].name);
					this.comboBox_cities.SelectedIndex=0;
				}
			}
			this.comboBox_orientation.SelectedIndex=(int)this.obj_options.window_orientation;
			this.comboBox_temp_unit.SelectedIndex=(int)this.obj_options.temp_unit;
			this.comboBox_wind_unit.SelectedIndex=(int)this.obj_options.wind_unit;
			this.comboBox_pressure_unit.SelectedIndex=(int)this.obj_options.pressure_unit;
			this.comboBox_length_unit.SelectedIndex=(int)this.obj_options.length_unit;
			this.comboBox_precipitation_height_unit.SelectedIndex=(int)this.obj_options.precipitation_height_unit;
			this.comboBox_height_unit.SelectedIndex=(int)this.obj_options.height_unit;
			this.comboBox_language.Text=this.obj_options.report_language;
			this.frm_details.set_language(this.exe_path+LANGUAGE_DIRECTORY+this.obj_options.report_language+".xml");

			this.checkBox_clouds.Checked=this.obj_options.show_clouds_in_satusbar;
			this.checkBox_temp.Checked=this.obj_options.show_temp_in_satusbar;
			this.checkBox_wind_dir.Checked=this.obj_options.show_wind_dir_in_satusbar;
			this.checkBox_wind_speed.Checked=this.obj_options.show_wind_speed_in_satusbar;
			this.checkBox_w_clouds.Checked=this.obj_options.show_clouds_in_window;
			this.checkBox_w_temp.Checked=this.obj_options.show_temp_in_window;
			this.checkBox_w_wind.Checked=this.obj_options.show_wind_in_window;
			this.checkBox_w_pressure.Checked=this.obj_options.show_pressure_in_window;
			this.checkBox_w_topmost.Checked=this.obj_options.top_most;
			this.trackBar_w_opacity.Value=(int)(this.obj_options.opacity*100);

			int index=this.comboBox_cities.FindStringExact(this.obj_options.city.country+" : "+this.obj_options.city.name);
			if (index<0)
				index=0;
			if (this.comboBox_cities.Items.Count>index)
				this.comboBox_cities.SelectedIndex=index;


			this.textBox_remote_host.Text=this.obj_options.host;
			this.textBox_remote_host_port.Text=this.obj_options.host_port.ToString();
			this.textBox_remote_host_directory.Text=this.obj_options.remote_dir;
			this.checkBox_use_proxy.Checked=this.obj_options.proxy_used;
			this.groupBox_proxy.Enabled=this.checkBox_use_proxy.Checked;
			this.textBox_proxy_ip.Text=this.obj_options.proxy_ip;
			this.textBox_proxy_port.Text=this.obj_options.proxy_port.ToString();
			this.textBox_update_interval.Text=this.obj_options.update_interval.ToString();
			this.textBox_retry_interval.Text=this.obj_options.retry_interval.ToString();
			index=(int)(this.obj_options.proxy_type-1);
			if (index<0)
				index=0;
			if (this.comboBox_proxy_type.Items.Count>index)
				this.comboBox_proxy_type.SelectedIndex=index;


			index=(int)(this.obj_options.parsing_type);
			if (this.comboBox_reply_type.Items.Count>index)
				this.comboBox_reply_type.SelectedIndex=index;

			System.Drawing.Text.InstalledFontCollection ifc = new System.Drawing.Text.InstalledFontCollection();

			index = -1;
			for (int i = 0; i < ifc.Families.Length; i++)
			{
				this.comboBox_notify_font_family.Items.Add(ifc.Families[i].Name);
				if ( ( index== -1 ) && (this.obj_options.notify_font_name == ifc.Families[i].Name) )
					index = i;
			}
			if (index==-1)
				index = 0;
			this.comboBox_notify_font_family.SelectedIndex = index;

			this.numeric_UpDown_notify_font_size.Value = this.obj_options.notify_font_size;
			this.checkBox_notify_font_bold.Checked = this.obj_options.notify_font_bold;
		}
		#endregion

		protected override void Dispose( bool disposing )
		{
			this.obj_metar_download.event_MetarDownload_Arrival-=new MetarDownload_Arrival_EventHandler(obj_metar_download_event_MetarDownload_Arrival);
			this.obj_metar_download.event_MetarDownload_Error-=new MetarDownload_Error_EventHandler(obj_metar_download_event_MetarDownload_Error);
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#region Windows Form Designer generated code
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FormMain));
			this.contextMenu_main = new System.Windows.Forms.ContextMenu();
			this.menuItem_hide_show = new System.Windows.Forms.MenuItem();
			this.menuItem_details = new System.Windows.Forms.MenuItem();
			this.menuItem_options = new System.Windows.Forms.MenuItem();
			this.menuItem_about = new System.Windows.Forms.MenuItem();
			this.menuItem_separator = new System.Windows.Forms.MenuItem();
			this.menuItem_exit = new System.Windows.Forms.MenuItem();
			this.button_apply = new System.Windows.Forms.Button();
			this.button_cancel = new System.Windows.Forms.Button();
			this.button_exit = new System.Windows.Forms.Button();
			this.tabControl_main = new System.Windows.Forms.TabControl();
			this.tabPage_general = new System.Windows.Forms.TabPage();
			this.groupBoxnotify_font = new System.Windows.Forms.GroupBox();
			this.comboBox_notify_font_family = new System.Windows.Forms.ComboBox();
			this.numeric_UpDown_notify_font_size = new System.Windows.Forms.NumericUpDown();
			this.checkBox_notify_font_bold = new System.Windows.Forms.CheckBox();
			this.label_notify_font_size = new System.Windows.Forms.Label();
			this.label_notify_font_family = new System.Windows.Forms.Label();
			this.groupBox_language = new System.Windows.Forms.GroupBox();
			this.comboBox_language = new System.Windows.Forms.ComboBox();
			this.groupBox_current_state = new System.Windows.Forms.GroupBox();
			this.label_current_state = new System.Windows.Forms.Label();
			this.label_city = new System.Windows.Forms.Label();
			this.comboBox_cities = new System.Windows.Forms.ComboBox();
			this.groupBox_Window = new System.Windows.Forms.GroupBox();
			this.comboBox_orientation = new System.Windows.Forms.ComboBox();
			this.label_orientation = new System.Windows.Forms.Label();
			this.checkBox_w_pressure = new System.Windows.Forms.CheckBox();
			this.label_window_opacity = new System.Windows.Forms.Label();
			this.trackBar_w_opacity = new System.Windows.Forms.TrackBar();
			this.checkBox_w_topmost = new System.Windows.Forms.CheckBox();
			this.checkBox_w_wind = new System.Windows.Forms.CheckBox();
			this.checkBox_w_clouds = new System.Windows.Forms.CheckBox();
			this.checkBox_w_temp = new System.Windows.Forms.CheckBox();
			this.groupBox_notifyicon = new System.Windows.Forms.GroupBox();
			this.checkBox_wind_speed = new System.Windows.Forms.CheckBox();
			this.checkBox_wind_dir = new System.Windows.Forms.CheckBox();
			this.checkBox_clouds = new System.Windows.Forms.CheckBox();
			this.checkBox_temp = new System.Windows.Forms.CheckBox();
			this.tabPage_units = new System.Windows.Forms.TabPage();
			this.comboBox_height_unit = new System.Windows.Forms.ComboBox();
			this.label_height_unit = new System.Windows.Forms.Label();
			this.comboBox_length_unit = new System.Windows.Forms.ComboBox();
			this.comboBox_precipitation_height_unit = new System.Windows.Forms.ComboBox();
			this.label_length_unit = new System.Windows.Forms.Label();
			this.label_precipitation_height_unit = new System.Windows.Forms.Label();
			this.comboBox_pressure_unit = new System.Windows.Forms.ComboBox();
			this.label_pressure_unit = new System.Windows.Forms.Label();
			this.comboBox_wind_unit = new System.Windows.Forms.ComboBox();
			this.comboBox_temp_unit = new System.Windows.Forms.ComboBox();
			this.label_temp_unit = new System.Windows.Forms.Label();
			this.label_wind_unit = new System.Windows.Forms.Label();
			this.tabPage_connection = new System.Windows.Forms.TabPage();
			this.comboBox_reply_type = new System.Windows.Forms.ComboBox();
			this.label_received_data_type = new System.Windows.Forms.Label();
			this.label_remote_host_directory = new System.Windows.Forms.Label();
			this.textBox_retry_interval = new System.Windows.Forms.TextBox();
			this.label_retry_interval = new System.Windows.Forms.Label();
			this.textBox_update_interval = new System.Windows.Forms.TextBox();
			this.label_update_interval = new System.Windows.Forms.Label();
			this.checkBox_use_proxy = new System.Windows.Forms.CheckBox();
			this.label_remote_host_port = new System.Windows.Forms.Label();
			this.textBox_remote_host_port = new System.Windows.Forms.TextBox();
			this.groupBox_proxy = new System.Windows.Forms.GroupBox();
			this.comboBox_proxy_type = new System.Windows.Forms.ComboBox();
			this.label_proxy_type = new System.Windows.Forms.Label();
			this.textBox_proxy_port = new System.Windows.Forms.TextBox();
			this.label_proxy_port = new System.Windows.Forms.Label();
			this.textBox_proxy_ip = new System.Windows.Forms.TextBox();
			this.label_proxy_ip = new System.Windows.Forms.Label();
			this.textBox_remote_host_directory = new System.Windows.Forms.TextBox();
			this.textBox_remote_host = new System.Windows.Forms.TextBox();
			this.label_remote_host_ip = new System.Windows.Forms.Label();
			this.tabControl_main.SuspendLayout();
			this.tabPage_general.SuspendLayout();
			this.groupBoxnotify_font.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.numeric_UpDown_notify_font_size)).BeginInit();
			this.groupBox_language.SuspendLayout();
			this.groupBox_current_state.SuspendLayout();
			this.groupBox_Window.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.trackBar_w_opacity)).BeginInit();
			this.groupBox_notifyicon.SuspendLayout();
			this.tabPage_units.SuspendLayout();
			this.tabPage_connection.SuspendLayout();
			this.groupBox_proxy.SuspendLayout();
			this.SuspendLayout();
			// 
			// contextMenu_main
			// 
			this.contextMenu_main.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																							 this.menuItem_hide_show,
																							 this.menuItem_details,
																							 this.menuItem_options,
																							 this.menuItem_about,
																							 this.menuItem_separator,
																							 this.menuItem_exit});
			// 
			// menuItem_hide_show
			// 
			this.menuItem_hide_show.Index = 0;
			this.menuItem_hide_show.Text = "Hide/Show";
			this.menuItem_hide_show.Click += new System.EventHandler(this.menuItem_hide_show_Click);
			// 
			// menuItem_details
			// 
			this.menuItem_details.Index = 1;
			this.menuItem_details.Text = "Details";
			this.menuItem_details.Click += new System.EventHandler(this.menuItem_details_Click);
			// 
			// menuItem_options
			// 
			this.menuItem_options.Index = 2;
			this.menuItem_options.Text = "Options";
			this.menuItem_options.Click += new System.EventHandler(this.menuItem_options_Click);
			// 
			// menuItem_about
			// 
			this.menuItem_about.Index = 3;
			this.menuItem_about.Text = "About";
			this.menuItem_about.Click += new System.EventHandler(this.menuItem_about_Click);
			// 
			// menuItem_separator
			// 
			this.menuItem_separator.Index = 4;
			this.menuItem_separator.Text = "-";
			// 
			// menuItem_exit
			// 
			this.menuItem_exit.Index = 5;
			this.menuItem_exit.Text = "Exit";
			this.menuItem_exit.Click += new System.EventHandler(this.menuItem_exit_Click);
			// 
			// button_apply
			// 
			this.button_apply.Location = new System.Drawing.Point(20, 349);
			this.button_apply.Name = "button_apply";
			this.button_apply.TabIndex = 8;
			this.button_apply.Text = "Apply";
			this.button_apply.Click += new System.EventHandler(this.button_apply_Click);
			// 
			// button_cancel
			// 
			this.button_cancel.Location = new System.Drawing.Point(108, 349);
			this.button_cancel.Name = "button_cancel";
			this.button_cancel.TabIndex = 14;
			this.button_cancel.Text = "Cancel";
			this.button_cancel.Click += new System.EventHandler(this.button_cancel_Click);
			// 
			// button_exit
			// 
			this.button_exit.Location = new System.Drawing.Point(196, 349);
			this.button_exit.Name = "button_exit";
			this.button_exit.TabIndex = 19;
			this.button_exit.Text = "Exit";
			this.button_exit.Click += new System.EventHandler(this.button_exit_Click);
			// 
			// tabControl_main
			// 
			this.tabControl_main.Controls.Add(this.tabPage_general);
			this.tabControl_main.Controls.Add(this.tabPage_units);
			this.tabControl_main.Controls.Add(this.tabPage_connection);
			this.tabControl_main.Dock = System.Windows.Forms.DockStyle.Top;
			this.tabControl_main.Location = new System.Drawing.Point(0, 0);
			this.tabControl_main.Name = "tabControl_main";
			this.tabControl_main.SelectedIndex = 0;
			this.tabControl_main.Size = new System.Drawing.Size(290, 346);
			this.tabControl_main.TabIndex = 25;
			// 
			// tabPage_general
			// 
			this.tabPage_general.Controls.Add(this.groupBoxnotify_font);
			this.tabPage_general.Controls.Add(this.groupBox_language);
			this.tabPage_general.Controls.Add(this.groupBox_current_state);
			this.tabPage_general.Controls.Add(this.label_city);
			this.tabPage_general.Controls.Add(this.comboBox_cities);
			this.tabPage_general.Controls.Add(this.groupBox_Window);
			this.tabPage_general.Controls.Add(this.groupBox_notifyicon);
			this.tabPage_general.Location = new System.Drawing.Point(4, 22);
			this.tabPage_general.Name = "tabPage_general";
			this.tabPage_general.Size = new System.Drawing.Size(282, 320);
			this.tabPage_general.TabIndex = 0;
			this.tabPage_general.Text = "General";
			// 
			// groupBoxnotify_font
			// 
			this.groupBoxnotify_font.Controls.Add(this.comboBox_notify_font_family);
			this.groupBoxnotify_font.Controls.Add(this.numeric_UpDown_notify_font_size);
			this.groupBoxnotify_font.Controls.Add(this.checkBox_notify_font_bold);
			this.groupBoxnotify_font.Controls.Add(this.label_notify_font_size);
			this.groupBoxnotify_font.Controls.Add(this.label_notify_font_family);
			this.groupBoxnotify_font.Location = new System.Drawing.Point(7, 249);
			this.groupBoxnotify_font.Name = "groupBoxnotify_font";
			this.groupBoxnotify_font.Size = new System.Drawing.Size(266, 67);
			this.groupBoxnotify_font.TabIndex = 31;
			this.groupBoxnotify_font.TabStop = false;
			this.groupBoxnotify_font.Text = "Notify Font";
			// 
			// comboBox_notify_font_family
			// 
			this.comboBox_notify_font_family.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_notify_font_family.Location = new System.Drawing.Point(88, 16);
			this.comboBox_notify_font_family.Name = "comboBox_notify_font_family";
			this.comboBox_notify_font_family.Size = new System.Drawing.Size(168, 21);
			this.comboBox_notify_font_family.TabIndex = 5;
			// 
			// numeric_UpDown_notify_font_size
			// 
			this.numeric_UpDown_notify_font_size.Location = new System.Drawing.Point(88, 40);
			this.numeric_UpDown_notify_font_size.Name = "numeric_UpDown_notify_font_size";
			this.numeric_UpDown_notify_font_size.Size = new System.Drawing.Size(40, 20);
			this.numeric_UpDown_notify_font_size.TabIndex = 4;
			this.numeric_UpDown_notify_font_size.Value = new System.Decimal(new int[] {
																						  10,
																						  0,
																						  0,
																						  0});
			// 
			// checkBox_notify_font_bold
			// 
			this.checkBox_notify_font_bold.Location = new System.Drawing.Point(176, 43);
			this.checkBox_notify_font_bold.Name = "checkBox_notify_font_bold";
			this.checkBox_notify_font_bold.Size = new System.Drawing.Size(75, 17);
			this.checkBox_notify_font_bold.TabIndex = 2;
			this.checkBox_notify_font_bold.Text = "bold";
			// 
			// label_notify_font_size
			// 
			this.label_notify_font_size.Location = new System.Drawing.Point(16, 43);
			this.label_notify_font_size.Name = "label_notify_font_size";
			this.label_notify_font_size.Size = new System.Drawing.Size(64, 16);
			this.label_notify_font_size.TabIndex = 1;
			this.label_notify_font_size.Text = "Size";
			// 
			// label_notify_font_family
			// 
			this.label_notify_font_family.Location = new System.Drawing.Point(16, 19);
			this.label_notify_font_family.Name = "label_notify_font_family";
			this.label_notify_font_family.Size = new System.Drawing.Size(64, 16);
			this.label_notify_font_family.TabIndex = 0;
			this.label_notify_font_family.Text = "Family";
			// 
			// groupBox_language
			// 
			this.groupBox_language.Controls.Add(this.comboBox_language);
			this.groupBox_language.Location = new System.Drawing.Point(7, 112);
			this.groupBox_language.Name = "groupBox_language";
			this.groupBox_language.Size = new System.Drawing.Size(120, 48);
			this.groupBox_language.TabIndex = 30;
			this.groupBox_language.TabStop = false;
			this.groupBox_language.Text = "Language";
			// 
			// comboBox_language
			// 
			this.comboBox_language.Location = new System.Drawing.Point(8, 16);
			this.comboBox_language.Name = "comboBox_language";
			this.comboBox_language.Size = new System.Drawing.Size(104, 21);
			this.comboBox_language.TabIndex = 0;
			// 
			// groupBox_current_state
			// 
			this.groupBox_current_state.Controls.Add(this.label_current_state);
			this.groupBox_current_state.Location = new System.Drawing.Point(8, 48);
			this.groupBox_current_state.Name = "groupBox_current_state";
			this.groupBox_current_state.Size = new System.Drawing.Size(120, 64);
			this.groupBox_current_state.TabIndex = 29;
			this.groupBox_current_state.TabStop = false;
			this.groupBox_current_state.Text = "Current State";
			// 
			// label_current_state
			// 
			this.label_current_state.Dock = System.Windows.Forms.DockStyle.Fill;
			this.label_current_state.Location = new System.Drawing.Point(3, 16);
			this.label_current_state.Name = "label_current_state";
			this.label_current_state.Size = new System.Drawing.Size(114, 45);
			this.label_current_state.TabIndex = 0;
			this.label_current_state.Text = "Stopped";
			this.label_current_state.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// label_city
			// 
			this.label_city.Location = new System.Drawing.Point(5, 8);
			this.label_city.Name = "label_city";
			this.label_city.Size = new System.Drawing.Size(120, 16);
			this.label_city.TabIndex = 28;
			this.label_city.Text = "Country : City";
			// 
			// comboBox_cities
			// 
			this.comboBox_cities.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_cities.Location = new System.Drawing.Point(5, 24);
			this.comboBox_cities.Name = "comboBox_cities";
			this.comboBox_cities.Size = new System.Drawing.Size(272, 21);
			this.comboBox_cities.TabIndex = 27;
			// 
			// groupBox_Window
			// 
			this.groupBox_Window.Controls.Add(this.comboBox_orientation);
			this.groupBox_Window.Controls.Add(this.label_orientation);
			this.groupBox_Window.Controls.Add(this.checkBox_w_pressure);
			this.groupBox_Window.Controls.Add(this.label_window_opacity);
			this.groupBox_Window.Controls.Add(this.trackBar_w_opacity);
			this.groupBox_Window.Controls.Add(this.checkBox_w_topmost);
			this.groupBox_Window.Controls.Add(this.checkBox_w_wind);
			this.groupBox_Window.Controls.Add(this.checkBox_w_clouds);
			this.groupBox_Window.Controls.Add(this.checkBox_w_temp);
			this.groupBox_Window.Location = new System.Drawing.Point(133, 48);
			this.groupBox_Window.Name = "groupBox_Window";
			this.groupBox_Window.Size = new System.Drawing.Size(144, 200);
			this.groupBox_Window.TabIndex = 26;
			this.groupBox_Window.TabStop = false;
			this.groupBox_Window.Text = "Window";
			// 
			// comboBox_orientation
			// 
			this.comboBox_orientation.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_orientation.Items.AddRange(new object[] {
																	  "Horizontal",
																	  "Vertical"});
			this.comboBox_orientation.Location = new System.Drawing.Point(8, 168);
			this.comboBox_orientation.Name = "comboBox_orientation";
			this.comboBox_orientation.Size = new System.Drawing.Size(128, 21);
			this.comboBox_orientation.TabIndex = 17;
			// 
			// label_orientation
			// 
			this.label_orientation.Location = new System.Drawing.Point(8, 152);
			this.label_orientation.Name = "label_orientation";
			this.label_orientation.Size = new System.Drawing.Size(128, 16);
			this.label_orientation.TabIndex = 16;
			this.label_orientation.Text = "Orientation";
			// 
			// checkBox_w_pressure
			// 
			this.checkBox_w_pressure.Location = new System.Drawing.Point(8, 64);
			this.checkBox_w_pressure.Name = "checkBox_w_pressure";
			this.checkBox_w_pressure.Size = new System.Drawing.Size(120, 16);
			this.checkBox_w_pressure.TabIndex = 11;
			this.checkBox_w_pressure.Text = "Pressure";
			// 
			// label_window_opacity
			// 
			this.label_window_opacity.Location = new System.Drawing.Point(8, 112);
			this.label_window_opacity.Name = "label_window_opacity";
			this.label_window_opacity.Size = new System.Drawing.Size(128, 16);
			this.label_window_opacity.TabIndex = 13;
			this.label_window_opacity.Text = "Window Opacity";
			// 
			// trackBar_w_opacity
			// 
			this.trackBar_w_opacity.AutoSize = false;
			this.trackBar_w_opacity.Location = new System.Drawing.Point(8, 128);
			this.trackBar_w_opacity.Maximum = 100;
			this.trackBar_w_opacity.Name = "trackBar_w_opacity";
			this.trackBar_w_opacity.Size = new System.Drawing.Size(128, 24);
			this.trackBar_w_opacity.TabIndex = 12;
			this.trackBar_w_opacity.Tag = "Opacity";
			this.trackBar_w_opacity.TickStyle = System.Windows.Forms.TickStyle.None;
			this.trackBar_w_opacity.Value = 80;
			// 
			// checkBox_w_topmost
			// 
			this.checkBox_w_topmost.Location = new System.Drawing.Point(8, 96);
			this.checkBox_w_topmost.Name = "checkBox_w_topmost";
			this.checkBox_w_topmost.Size = new System.Drawing.Size(128, 16);
			this.checkBox_w_topmost.TabIndex = 12;
			this.checkBox_w_topmost.Text = "Top Most Window";
			// 
			// checkBox_w_wind
			// 
			this.checkBox_w_wind.Location = new System.Drawing.Point(8, 48);
			this.checkBox_w_wind.Name = "checkBox_w_wind";
			this.checkBox_w_wind.Size = new System.Drawing.Size(120, 16);
			this.checkBox_w_wind.TabIndex = 10;
			this.checkBox_w_wind.Text = "Wind";
			// 
			// checkBox_w_clouds
			// 
			this.checkBox_w_clouds.Location = new System.Drawing.Point(8, 16);
			this.checkBox_w_clouds.Name = "checkBox_w_clouds";
			this.checkBox_w_clouds.Size = new System.Drawing.Size(120, 16);
			this.checkBox_w_clouds.TabIndex = 8;
			this.checkBox_w_clouds.Text = "Clouds";
			// 
			// checkBox_w_temp
			// 
			this.checkBox_w_temp.Location = new System.Drawing.Point(8, 32);
			this.checkBox_w_temp.Name = "checkBox_w_temp";
			this.checkBox_w_temp.Size = new System.Drawing.Size(120, 16);
			this.checkBox_w_temp.TabIndex = 9;
			this.checkBox_w_temp.Text = "Temperature";
			// 
			// groupBox_notifyicon
			// 
			this.groupBox_notifyicon.Controls.Add(this.checkBox_wind_speed);
			this.groupBox_notifyicon.Controls.Add(this.checkBox_wind_dir);
			this.groupBox_notifyicon.Controls.Add(this.checkBox_clouds);
			this.groupBox_notifyicon.Controls.Add(this.checkBox_temp);
			this.groupBox_notifyicon.Location = new System.Drawing.Point(7, 159);
			this.groupBox_notifyicon.Name = "groupBox_notifyicon";
			this.groupBox_notifyicon.Size = new System.Drawing.Size(120, 88);
			this.groupBox_notifyicon.TabIndex = 25;
			this.groupBox_notifyicon.TabStop = false;
			this.groupBox_notifyicon.Text = "Notify Icons";
			// 
			// checkBox_wind_speed
			// 
			this.checkBox_wind_speed.Location = new System.Drawing.Point(8, 64);
			this.checkBox_wind_speed.Name = "checkBox_wind_speed";
			this.checkBox_wind_speed.Size = new System.Drawing.Size(104, 16);
			this.checkBox_wind_speed.TabIndex = 7;
			this.checkBox_wind_speed.Text = "Wind Speed";
			// 
			// checkBox_wind_dir
			// 
			this.checkBox_wind_dir.Location = new System.Drawing.Point(8, 48);
			this.checkBox_wind_dir.Name = "checkBox_wind_dir";
			this.checkBox_wind_dir.Size = new System.Drawing.Size(104, 16);
			this.checkBox_wind_dir.TabIndex = 6;
			this.checkBox_wind_dir.Text = "Wind Direction";
			// 
			// checkBox_clouds
			// 
			this.checkBox_clouds.Location = new System.Drawing.Point(8, 16);
			this.checkBox_clouds.Name = "checkBox_clouds";
			this.checkBox_clouds.Size = new System.Drawing.Size(104, 16);
			this.checkBox_clouds.TabIndex = 4;
			this.checkBox_clouds.Text = "Clouds";
			// 
			// checkBox_temp
			// 
			this.checkBox_temp.Location = new System.Drawing.Point(8, 32);
			this.checkBox_temp.Name = "checkBox_temp";
			this.checkBox_temp.Size = new System.Drawing.Size(104, 16);
			this.checkBox_temp.TabIndex = 5;
			this.checkBox_temp.Text = "Temperature";
			// 
			// tabPage_units
			// 
			this.tabPage_units.Controls.Add(this.comboBox_height_unit);
			this.tabPage_units.Controls.Add(this.label_height_unit);
			this.tabPage_units.Controls.Add(this.comboBox_length_unit);
			this.tabPage_units.Controls.Add(this.comboBox_precipitation_height_unit);
			this.tabPage_units.Controls.Add(this.label_length_unit);
			this.tabPage_units.Controls.Add(this.label_precipitation_height_unit);
			this.tabPage_units.Controls.Add(this.comboBox_pressure_unit);
			this.tabPage_units.Controls.Add(this.label_pressure_unit);
			this.tabPage_units.Controls.Add(this.comboBox_wind_unit);
			this.tabPage_units.Controls.Add(this.comboBox_temp_unit);
			this.tabPage_units.Controls.Add(this.label_temp_unit);
			this.tabPage_units.Controls.Add(this.label_wind_unit);
			this.tabPage_units.Location = new System.Drawing.Point(4, 22);
			this.tabPage_units.Name = "tabPage_units";
			this.tabPage_units.Size = new System.Drawing.Size(282, 320);
			this.tabPage_units.TabIndex = 1;
			this.tabPage_units.Text = "Units";
			// 
			// comboBox_height_unit
			// 
			this.comboBox_height_unit.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_height_unit.ItemHeight = 13;
			this.comboBox_height_unit.Items.AddRange(new object[] {
																	  "FEET",
																	  "KM",
																	  "METER",
																	  "MILES"});
			this.comboBox_height_unit.Location = new System.Drawing.Point(144, 224);
			this.comboBox_height_unit.Name = "comboBox_height_unit";
			this.comboBox_height_unit.Size = new System.Drawing.Size(136, 21);
			this.comboBox_height_unit.TabIndex = 46;
			// 
			// label_height_unit
			// 
			this.label_height_unit.Location = new System.Drawing.Point(144, 203);
			this.label_height_unit.Name = "label_height_unit";
			this.label_height_unit.Size = new System.Drawing.Size(128, 16);
			this.label_height_unit.TabIndex = 45;
			this.label_height_unit.Text = "Height";
			// 
			// comboBox_length_unit
			// 
			this.comboBox_length_unit.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_length_unit.ItemHeight = 13;
			this.comboBox_length_unit.Items.AddRange(new object[] {
																	  "FEET",
																	  "KM",
																	  "METER",
																	  "MILES"});
			this.comboBox_length_unit.Location = new System.Drawing.Point(144, 136);
			this.comboBox_length_unit.Name = "comboBox_length_unit";
			this.comboBox_length_unit.Size = new System.Drawing.Size(136, 21);
			this.comboBox_length_unit.TabIndex = 44;
			// 
			// comboBox_precipitation_height_unit
			// 
			this.comboBox_precipitation_height_unit.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_precipitation_height_unit.ItemHeight = 13;
			this.comboBox_precipitation_height_unit.Items.AddRange(new object[] {
																					"INCH",
																					"MM"});
			this.comboBox_precipitation_height_unit.Location = new System.Drawing.Point(0, 224);
			this.comboBox_precipitation_height_unit.Name = "comboBox_precipitation_height_unit";
			this.comboBox_precipitation_height_unit.Size = new System.Drawing.Size(136, 21);
			this.comboBox_precipitation_height_unit.TabIndex = 43;
			// 
			// label_length_unit
			// 
			this.label_length_unit.Location = new System.Drawing.Point(144, 117);
			this.label_length_unit.Name = "label_length_unit";
			this.label_length_unit.Size = new System.Drawing.Size(128, 16);
			this.label_length_unit.TabIndex = 42;
			this.label_length_unit.Text = "Length";
			// 
			// label_precipitation_height_unit
			// 
			this.label_precipitation_height_unit.Location = new System.Drawing.Point(0, 203);
			this.label_precipitation_height_unit.Name = "label_precipitation_height_unit";
			this.label_precipitation_height_unit.Size = new System.Drawing.Size(144, 16);
			this.label_precipitation_height_unit.TabIndex = 41;
			this.label_precipitation_height_unit.Text = "Precipitation Height";
			// 
			// comboBox_pressure_unit
			// 
			this.comboBox_pressure_unit.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_pressure_unit.ItemHeight = 13;
			this.comboBox_pressure_unit.Items.AddRange(new object[] {
																		"INHG",
																		"MMHG",
																		"HPA",
																		"ATM"});
			this.comboBox_pressure_unit.Location = new System.Drawing.Point(0, 138);
			this.comboBox_pressure_unit.Name = "comboBox_pressure_unit";
			this.comboBox_pressure_unit.Size = new System.Drawing.Size(136, 21);
			this.comboBox_pressure_unit.TabIndex = 40;
			// 
			// label_pressure_unit
			// 
			this.label_pressure_unit.Location = new System.Drawing.Point(0, 120);
			this.label_pressure_unit.Name = "label_pressure_unit";
			this.label_pressure_unit.Size = new System.Drawing.Size(128, 16);
			this.label_pressure_unit.TabIndex = 39;
			this.label_pressure_unit.Text = "Pressure";
			// 
			// comboBox_wind_unit
			// 
			this.comboBox_wind_unit.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_wind_unit.ItemHeight = 13;
			this.comboBox_wind_unit.Items.AddRange(new object[] {
																	"Meter/Second",
																	"KiloMeter/Hour",
																	"Miles/Hour",
																	"Knots",
																	"Beaufort"});
			this.comboBox_wind_unit.Location = new System.Drawing.Point(144, 53);
			this.comboBox_wind_unit.Name = "comboBox_wind_unit";
			this.comboBox_wind_unit.Size = new System.Drawing.Size(136, 21);
			this.comboBox_wind_unit.TabIndex = 38;
			// 
			// comboBox_temp_unit
			// 
			this.comboBox_temp_unit.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_temp_unit.ItemHeight = 13;
			this.comboBox_temp_unit.Items.AddRange(new object[] {
																	"Fahrenheit",
																	"Celcius"});
			this.comboBox_temp_unit.Location = new System.Drawing.Point(0, 53);
			this.comboBox_temp_unit.Name = "comboBox_temp_unit";
			this.comboBox_temp_unit.Size = new System.Drawing.Size(136, 21);
			this.comboBox_temp_unit.TabIndex = 37;
			// 
			// label_temp_unit
			// 
			this.label_temp_unit.Location = new System.Drawing.Point(8, 32);
			this.label_temp_unit.Name = "label_temp_unit";
			this.label_temp_unit.Size = new System.Drawing.Size(120, 16);
			this.label_temp_unit.TabIndex = 36;
			this.label_temp_unit.Text = "Temperature";
			// 
			// label_wind_unit
			// 
			this.label_wind_unit.Location = new System.Drawing.Point(144, 32);
			this.label_wind_unit.Name = "label_wind_unit";
			this.label_wind_unit.Size = new System.Drawing.Size(136, 16);
			this.label_wind_unit.TabIndex = 35;
			this.label_wind_unit.Text = "Wind Speed";
			// 
			// tabPage_connection
			// 
			this.tabPage_connection.Controls.Add(this.comboBox_reply_type);
			this.tabPage_connection.Controls.Add(this.label_received_data_type);
			this.tabPage_connection.Controls.Add(this.label_remote_host_directory);
			this.tabPage_connection.Controls.Add(this.textBox_retry_interval);
			this.tabPage_connection.Controls.Add(this.label_retry_interval);
			this.tabPage_connection.Controls.Add(this.textBox_update_interval);
			this.tabPage_connection.Controls.Add(this.label_update_interval);
			this.tabPage_connection.Controls.Add(this.checkBox_use_proxy);
			this.tabPage_connection.Controls.Add(this.label_remote_host_port);
			this.tabPage_connection.Controls.Add(this.textBox_remote_host_port);
			this.tabPage_connection.Controls.Add(this.groupBox_proxy);
			this.tabPage_connection.Controls.Add(this.textBox_remote_host_directory);
			this.tabPage_connection.Controls.Add(this.textBox_remote_host);
			this.tabPage_connection.Controls.Add(this.label_remote_host_ip);
			this.tabPage_connection.Location = new System.Drawing.Point(4, 22);
			this.tabPage_connection.Name = "tabPage_connection";
			this.tabPage_connection.Size = new System.Drawing.Size(282, 320);
			this.tabPage_connection.TabIndex = 2;
			this.tabPage_connection.Text = "Connection";
			// 
			// comboBox_reply_type
			// 
			this.comboBox_reply_type.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_reply_type.Items.AddRange(new object[] {
																	 "Text",
																	 "XML"});
			this.comboBox_reply_type.Location = new System.Drawing.Point(192, 96);
			this.comboBox_reply_type.Name = "comboBox_reply_type";
			this.comboBox_reply_type.Size = new System.Drawing.Size(80, 21);
			this.comboBox_reply_type.TabIndex = 13;
			// 
			// label_received_data_type
			// 
			this.label_received_data_type.Location = new System.Drawing.Point(8, 101);
			this.label_received_data_type.Name = "label_received_data_type";
			this.label_received_data_type.Size = new System.Drawing.Size(168, 16);
			this.label_received_data_type.TabIndex = 12;
			this.label_received_data_type.Text = "Received Data Type";
			// 
			// label_remote_host_directory
			// 
			this.label_remote_host_directory.Location = new System.Drawing.Point(8, 31);
			this.label_remote_host_directory.Name = "label_remote_host_directory";
			this.label_remote_host_directory.Size = new System.Drawing.Size(152, 16);
			this.label_remote_host_directory.TabIndex = 4;
			this.label_remote_host_directory.Text = "Remote Directory";
			this.label_remote_host_directory.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// textBox_retry_interval
			// 
			this.textBox_retry_interval.Location = new System.Drawing.Point(224, 152);
			this.textBox_retry_interval.Name = "textBox_retry_interval";
			this.textBox_retry_interval.Size = new System.Drawing.Size(48, 20);
			this.textBox_retry_interval.TabIndex = 11;
			this.textBox_retry_interval.Text = "300";
			// 
			// label_retry_interval
			// 
			this.label_retry_interval.Location = new System.Drawing.Point(8, 155);
			this.label_retry_interval.Name = "label_retry_interval";
			this.label_retry_interval.Size = new System.Drawing.Size(200, 20);
			this.label_retry_interval.TabIndex = 10;
			this.label_retry_interval.Text = "Retry Interval (in sec) in case of error";
			this.label_retry_interval.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// textBox_update_interval
			// 
			this.textBox_update_interval.Location = new System.Drawing.Point(224, 131);
			this.textBox_update_interval.Name = "textBox_update_interval";
			this.textBox_update_interval.Size = new System.Drawing.Size(48, 20);
			this.textBox_update_interval.TabIndex = 9;
			this.textBox_update_interval.Text = "3600";
			// 
			// label_update_interval
			// 
			this.label_update_interval.Location = new System.Drawing.Point(8, 131);
			this.label_update_interval.Name = "label_update_interval";
			this.label_update_interval.Size = new System.Drawing.Size(200, 20);
			this.label_update_interval.TabIndex = 8;
			this.label_update_interval.Text = "Update Interval (in sec)";
			this.label_update_interval.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// checkBox_use_proxy
			// 
			this.checkBox_use_proxy.Location = new System.Drawing.Point(8, 189);
			this.checkBox_use_proxy.Name = "checkBox_use_proxy";
			this.checkBox_use_proxy.Size = new System.Drawing.Size(160, 16);
			this.checkBox_use_proxy.TabIndex = 6;
			this.checkBox_use_proxy.Text = "Use proxy";
			this.checkBox_use_proxy.Visible = false;
			this.checkBox_use_proxy.CheckedChanged += new System.EventHandler(this.checkBox_use_proxy_CheckedChanged);
			// 
			// label_remote_host_port
			// 
			this.label_remote_host_port.Location = new System.Drawing.Point(8, 70);
			this.label_remote_host_port.Name = "label_remote_host_port";
			this.label_remote_host_port.Size = new System.Drawing.Size(72, 24);
			this.label_remote_host_port.TabIndex = 2;
			this.label_remote_host_port.Text = "Remote Port";
			this.label_remote_host_port.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.label_remote_host_port.Visible = false;
			// 
			// textBox_remote_host_port
			// 
			this.textBox_remote_host_port.Location = new System.Drawing.Point(80, 70);
			this.textBox_remote_host_port.Name = "textBox_remote_host_port";
			this.textBox_remote_host_port.Size = new System.Drawing.Size(48, 20);
			this.textBox_remote_host_port.TabIndex = 3;
			this.textBox_remote_host_port.Text = "80";
			this.textBox_remote_host_port.Visible = false;
			// 
			// groupBox_proxy
			// 
			this.groupBox_proxy.Controls.Add(this.comboBox_proxy_type);
			this.groupBox_proxy.Controls.Add(this.label_proxy_type);
			this.groupBox_proxy.Controls.Add(this.textBox_proxy_port);
			this.groupBox_proxy.Controls.Add(this.label_proxy_port);
			this.groupBox_proxy.Controls.Add(this.textBox_proxy_ip);
			this.groupBox_proxy.Controls.Add(this.label_proxy_ip);
			this.groupBox_proxy.Location = new System.Drawing.Point(24, 205);
			this.groupBox_proxy.Name = "groupBox_proxy";
			this.groupBox_proxy.Size = new System.Drawing.Size(248, 88);
			this.groupBox_proxy.TabIndex = 7;
			this.groupBox_proxy.TabStop = false;
			this.groupBox_proxy.Visible = false;
			// 
			// comboBox_proxy_type
			// 
			this.comboBox_proxy_type.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBox_proxy_type.Items.AddRange(new object[] {
																	 "HTTP GET",
																	 "HTTP CONNECT",
																	 "SOCKS 4",
																	 "SOCKS 5"});
			this.comboBox_proxy_type.Location = new System.Drawing.Point(88, 16);
			this.comboBox_proxy_type.Name = "comboBox_proxy_type";
			this.comboBox_proxy_type.Size = new System.Drawing.Size(128, 21);
			this.comboBox_proxy_type.TabIndex = 9;
			this.comboBox_proxy_type.Visible = false;
			// 
			// label_proxy_type
			// 
			this.label_proxy_type.Location = new System.Drawing.Point(16, 16);
			this.label_proxy_type.Name = "label_proxy_type";
			this.label_proxy_type.Size = new System.Drawing.Size(72, 20);
			this.label_proxy_type.TabIndex = 8;
			this.label_proxy_type.Text = "Proxy Type";
			this.label_proxy_type.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.label_proxy_type.Visible = false;
			// 
			// textBox_proxy_port
			// 
			this.textBox_proxy_port.Location = new System.Drawing.Point(88, 64);
			this.textBox_proxy_port.Name = "textBox_proxy_port";
			this.textBox_proxy_port.Size = new System.Drawing.Size(48, 20);
			this.textBox_proxy_port.TabIndex = 7;
			this.textBox_proxy_port.Text = "80";
			this.textBox_proxy_port.Visible = false;
			// 
			// label_proxy_port
			// 
			this.label_proxy_port.Location = new System.Drawing.Point(16, 64);
			this.label_proxy_port.Name = "label_proxy_port";
			this.label_proxy_port.Size = new System.Drawing.Size(72, 20);
			this.label_proxy_port.TabIndex = 6;
			this.label_proxy_port.Text = "Proxy Port";
			this.label_proxy_port.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.label_proxy_port.Visible = false;
			// 
			// textBox_proxy_ip
			// 
			this.textBox_proxy_ip.Location = new System.Drawing.Point(88, 40);
			this.textBox_proxy_ip.Name = "textBox_proxy_ip";
			this.textBox_proxy_ip.Size = new System.Drawing.Size(128, 20);
			this.textBox_proxy_ip.TabIndex = 5;
			this.textBox_proxy_ip.Text = "tgftp.nws.noaa.gov";
			this.textBox_proxy_ip.Visible = false;
			// 
			// label_proxy_ip
			// 
			this.label_proxy_ip.Location = new System.Drawing.Point(16, 40);
			this.label_proxy_ip.Name = "label_proxy_ip";
			this.label_proxy_ip.Size = new System.Drawing.Size(72, 20);
			this.label_proxy_ip.TabIndex = 4;
			this.label_proxy_ip.Text = "Proxy IP";
			this.label_proxy_ip.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.label_proxy_ip.Visible = false;
			// 
			// textBox_remote_host_directory
			// 
			this.textBox_remote_host_directory.Location = new System.Drawing.Point(8, 47);
			this.textBox_remote_host_directory.Name = "textBox_remote_host_directory";
			this.textBox_remote_host_directory.Size = new System.Drawing.Size(264, 20);
			this.textBox_remote_host_directory.TabIndex = 5;
			this.textBox_remote_host_directory.Text = "/data/observations/metar/stations/%METAR_NAME%.TXT";
			// 
			// textBox_remote_host
			// 
			this.textBox_remote_host.Location = new System.Drawing.Point(80, 8);
			this.textBox_remote_host.Name = "textBox_remote_host";
			this.textBox_remote_host.Size = new System.Drawing.Size(160, 20);
			this.textBox_remote_host.TabIndex = 1;
			this.textBox_remote_host.Text = "tgftp.nws.noaa.gov";
			// 
			// label_remote_host_ip
			// 
			this.label_remote_host_ip.Location = new System.Drawing.Point(8, 8);
			this.label_remote_host_ip.Name = "label_remote_host_ip";
			this.label_remote_host_ip.Size = new System.Drawing.Size(72, 20);
			this.label_remote_host_ip.TabIndex = 0;
			this.label_remote_host_ip.Text = "Remote Host";
			this.label_remote_host_ip.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// FormMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(290, 375);
			this.Controls.Add(this.tabControl_main);
			this.Controls.Add(this.button_exit);
			this.Controls.Add(this.button_cancel);
			this.Controls.Add(this.button_apply);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "FormMain";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Options";
			this.TopMost = true;
			this.WindowState = System.Windows.Forms.FormWindowState.Minimized;
			this.Closing += new System.ComponentModel.CancelEventHandler(this.FormMain_Closing);
			this.Load += new System.EventHandler(this.FormMain_Load);
			this.Activated += new System.EventHandler(this.FormMain_Activated);
			this.tabControl_main.ResumeLayout(false);
			this.tabPage_general.ResumeLayout(false);
			this.groupBoxnotify_font.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.numeric_UpDown_notify_font_size)).EndInit();
			this.groupBox_language.ResumeLayout(false);
			this.groupBox_current_state.ResumeLayout(false);
			this.groupBox_Window.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.trackBar_w_opacity)).EndInit();
			this.groupBox_notifyicon.ResumeLayout(false);
			this.tabPage_units.ResumeLayout(false);
			this.tabPage_connection.ResumeLayout(false);
			this.groupBox_proxy.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		public void set_frm_weatherview_options_position(int x,int y)
		{
			this.obj_options.window_pos_x=x;
			this.obj_options.window_pos_y=y;
			this.obj_options.save(this.exe_path+CONFIG_FILENAME);
		}

		private void show_ordered_notify_icons(int[] visible_item_number)
		{
			int cnt;
			// the left pos one should be put in last position
			// hide already visibled icons to show them again in a view to respect icon order
			for (cnt=notify_icon_number-1;cnt>-1;cnt--)
				this.array_NotifyIcon[cnt].Visible=false;

			// show visible item
			int cnt2;
			if (visible_item_number==null)
				return;
			if (visible_item_number.Length==0)
				return;
			for (cnt=notify_icon_number-1;cnt>-1;cnt--)
			{
				for (cnt2=visible_item_number.Length-1;cnt2>-1;cnt2--)
				{
					if (visible_item_number[cnt2]==cnt)
					{
						this.array_NotifyIcon[cnt].Visible=true;
						break;
					}
				}
			}
		}


		#region drawstring_in_notify_icon
		private void drawstring_in_notify_icon(string text,NotifyIcon notify_icon)
		{
			// Create a graphics instance that draws to a bitmap
			Bitmap bitmap = new Bitmap( 16, 16,System.Drawing.Imaging.PixelFormat.Format32bppArgb);

			Graphics graphics = Graphics.FromImage(bitmap);

			// FONT
			System.Drawing.FontStyle font_style;
			if (this.obj_options.notify_font_bold)
				font_style = System.Drawing.FontStyle.Bold;
			else
				font_style = System.Drawing.FontStyle.Regular;

			System.Drawing.Font drawFont = new System.Drawing.Font(this.obj_options.notify_font_name, (float)this.obj_options.notify_font_size, font_style);

			// TEXT
			string drawString = text;

			SizeF drawStringSize = new SizeF();
			//Measure the Copyright string in this Font
			drawStringSize = graphics.MeasureString(drawString, drawFont);

			// BRUSH
			System.Drawing.SolidBrush drawBrush = new System.Drawing.SolidBrush(Color.FromArgb( 255, 255, 255, 255));
			// center text in icon image
			graphics.DrawString(drawString, drawFont, drawBrush, 16/2-drawStringSize.Width/2,16/2-drawStringSize.Height/2);

			if (notify_icon.Icon!=null)
				notify_icon.Icon.Dispose();

			System.IntPtr ptIco=bitmap.GetHicon();
			notify_icon.Icon=System.Drawing.Icon.FromHandle(ptIco);

			// Destroy the Icon, since the form creates
			// its own copy of the icon.
			DestroyIcon(ptIco);

			drawFont.Dispose();
			drawBrush.Dispose();
			graphics.Dispose();
			bitmap.Dispose();
		}
		#endregion

		public delegate void obj_metar_download_event_MetarDownload_Arrival_thread_Handler(ClassMetarDownload sender, EventArgs_StringData e);
		private void obj_metar_download_event_MetarDownload_Arrival(ClassMetarDownload sender, EventArgs_StringData e)
		{
			this.Invoke(new obj_metar_download_event_MetarDownload_Arrival_thread_Handler(obj_metar_download_event_MetarDownload_Arrival_thread_safe), new Object[] {sender,e});
		}
		public delegate void obj_metar_download_event_MetarDownload_Error_thread_Handler(ClassMetarDownload sender, EventArgs_Exception e);
		private void obj_metar_download_event_MetarDownload_Error(ClassMetarDownload sender, EventArgs_Exception e)
		{
			this.Invoke(new obj_metar_download_event_MetarDownload_Error_thread_Handler(obj_metar_download_event_MetarDownload_Error_thread_safe), new Object[] {sender,e});
		}
		private void obj_metar_download_event_MetarDownload_Arrival_thread_safe(ClassMetarDownload sender, EventArgs_StringData e)
		{
			this.update_information(e.data);
		}

		private void obj_metar_download_event_MetarDownload_Error_thread_safe(ClassMetarDownload sender, EventArgs_Exception e)
		{
			MessageBox.Show(e.exception.Message,"Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
			// show no data imgs (by default obj_metar.clouds,obj_metar.wind and obj_metar.temperature return no data imgs)
			this.update_information("");
			/////////////////////////////////////////////////
			// for debug only if no internet connection
			/////////////////////////////////////////////////
			// string raw_metar="2004/11/28 10:00\r\nTFFR 061800Z 06010KT 010V090 9999 BKN026 28/20 Q1014 TEMPO 6000 SHRA\r\n";
			// this.update_information(raw_metar);
			/////////////////////////////////////////////////
		}

		#region update_information
		private void update_information(string str_metar)
		{
			Bitmap bitmap;
			bool b_empty=(str_metar=="");
			this.last_raw_metar=str_metar;
			ClassMetar obj_metar=new ClassMetar();
			obj_metar.decode_metar(str_metar);

			string str_img_clouds = "clouds_nodata.png";
			string str_img_wind = "wind_nodata.png";
			string str_img_temp_term = "temp_nodata.png";
			if (!b_empty)
			{
				str_img_clouds = obj_metar.clouds_array[0].get_sky_image(obj_metar.str_metar);
				str_img_wind = obj_metar.wind.get_winddir_image();
				str_img_temp_term = obj_metar.temperature.get_temp_image();
			}

			// switch options.temperature
			string str_temp="??";
			string str_temp_unit="";
			string str="";
			if (obj_metar.temperature.current.is_set())
			{
				str=obj_metar.temperature.current.get_string((ClassTemperature.UNITS)this.obj_options.temp_unit,false);
				str_temp=str.Substring(0,str.Length-1);
				str_temp_unit=str.Substring(str.Length-1);
			}

			// switch options.wind_speed
			string str_wind_speed="??";
			string str_wind_unit="";
			if (!obj_metar.wind.b_no_data)
			{
				switch(this.obj_options.wind_unit)
				{
					case ClassOptions.WIND_UNITS.BEAUFORT:
						str_wind_speed=obj_metar.wind.speed.Beaufort.ToString("0");
						str_wind_unit="B";
						break;
					case ClassOptions.WIND_UNITS.MS:
						if (obj_metar.wind.speed.Ms==Math.Round(obj_metar.wind.speed.Ms))// integer
							str_wind_speed=Math.Round(obj_metar.wind.speed.Ms).ToString("0");
						else
							str_wind_speed=Math.Round(obj_metar.wind.speed.Ms,1).ToString("0.0");
						str_wind_unit="Ms";
						break;
					case ClassOptions.WIND_UNITS.MH:
						if (obj_metar.wind.speed.Mileshours==Math.Round(obj_metar.wind.speed.Mileshours))// integer
							str_wind_speed=Math.Round(obj_metar.wind.speed.Mileshours).ToString("0");
						else
							str_wind_speed=Math.Round(obj_metar.wind.speed.Mileshours,1).ToString("0.0");
						str_wind_unit="Mh";
						break;
					case ClassOptions.WIND_UNITS.KNOTS:
						if (obj_metar.wind.speed.Knots==Math.Round(obj_metar.wind.speed.Knots))// integer
							str_wind_speed=Math.Round(obj_metar.wind.speed.Knots).ToString("0");
						else
							str_wind_speed=Math.Round(obj_metar.wind.speed.Knots,1).ToString("0.0");
						str_wind_unit="K";
						break;
					case ClassOptions.WIND_UNITS.KMH:
						str_wind_speed=Math.Round(obj_metar.wind.speed.Kmh,0).ToString("0");
						str_wind_unit="Kmh";
						break;
				}
			}
			int pos=str_wind_speed.IndexOf(".");
			if ((str_wind_speed.Length>4)&&(pos>0))
				str_wind_speed=str_wind_speed.Substring(0,pos-1);
			// switch pressure unit
			string str_pressure="";
			if (!obj_metar.pressure.b_no_data)
			{
				switch(this.obj_options.pressure_unit)
				{
					case ClassOptions.PRESSURE_UNITS.ATM:
						str_pressure=Math.Round(obj_metar.pressure.Atm,0).ToString("0")+" Atm";
						break;
					case ClassOptions.PRESSURE_UNITS.HPA:
						str_pressure=Math.Round(obj_metar.pressure.Hpa,0).ToString("0")+" HPa";
						break;
					case ClassOptions.PRESSURE_UNITS.INHG:
						str_pressure=Math.Round(obj_metar.pressure.Inhg,2).ToString("0.00")+" Ingh";
						break;
					case ClassOptions.PRESSURE_UNITS.MMHG:
						str_pressure=Math.Round(obj_metar.pressure.Mmhg,0).ToString("0")+" Mmhg";
						break;
				}
			}

			// Update form weather view
			// weather
			this.frm_weather_view.set_img_weather(this.exe_path+IMG_DIRECTORY+str_img_clouds);
			// temp
			this.frm_weather_view.set_temp(str_temp+str_temp_unit);
			this.frm_weather_view.set_img_temp(IMG_DIRECTORY+str_img_temp_term);
			// wind
			this.frm_weather_view.set_wind(str_wind_speed+" "+str_wind_unit);
			this.frm_weather_view.set_img_wind(this.exe_path+IMG_DIRECTORY+str_img_wind);
			// city
			if (this.obj_options.city.country!="")
				this.frm_weather_view.set_city(this.obj_options.city.name+" ("+this.obj_options.city.country+")");
			else
				this.frm_weather_view.set_city(this.obj_options.city.name);
			// pressure
			this.frm_weather_view.set_pressure(str_pressure);

			this.set_notify_icon_text();

			if (str_img_wind!="")
				// tips : str_img_wind contains the direction of wind so use it ;)
				array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_IMG].Text=this.obj_interface_language.wind_direction+" ("+str_img_wind.Substring(0,str_img_wind.Length-4)+")";
			else
				array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_IMG].Text=this.obj_interface_language.wind_direction;

			array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_DATA].Visible=this.obj_options.show_wind_speed_in_satusbar;
			array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_IMG].Visible=this.obj_options.show_wind_dir_in_satusbar;
			array_NotifyIcon[(int)NOTIFY_ICON_POS.TEMP].Visible=this.obj_options.show_temp_in_satusbar;
			array_NotifyIcon[(int)NOTIFY_ICON_POS.WEATHER].Visible=this.obj_options.show_clouds_in_satusbar;
			// update notify icons (must be reverse from the wanted position in case of already visible with no image)
			this.drawstring_in_notify_icon(str_wind_speed,this.array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_DATA]);
			if (str_img_wind!="")
			{
				bitmap=new Bitmap(this.exe_path+IMG_DIRECTORY+str_img_wind);
				// Bitmap bmp=this.GetThumb(bitmap,16,16);
				Bitmap bmp=this.bitmap_resize(bitmap,0,0,16,16);	
				if (array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_IMG].Icon!=null)
					array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_IMG].Icon.Dispose();

				System.IntPtr ptIco=bmp.GetHicon();

				array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_IMG].Icon=System.Drawing.Icon.FromHandle(ptIco);

				// Destroy the Icon, since the form creates
				// its own copy of the icon.
				DestroyIcon(ptIco);
				bmp.Dispose();
				bitmap.Dispose();
			}

			// remove  if str_temp is too long
			if (str_temp.Length > 3)
				str_temp = str_temp.Substring(0, 3);
			this.drawstring_in_notify_icon(str_temp, this.array_NotifyIcon[(int)NOTIFY_ICON_POS.TEMP]);

			if (str_img_clouds!="")
			{
				// if imgs are 80*50 crop them to 50*50 to avoid distortion
				bitmap=new Bitmap(this.exe_path+IMG_DIRECTORY+str_img_clouds);
				// Bitmap bmpCrop=this.bitmap_crop(bitmap,15,0,50,50);
				// Bitmap bmp=this.GetThumb(bmpCrop,16,16);
				Bitmap bmp=this.bitmap_resize(bitmap,0,0,16,16);			
                
				if (array_NotifyIcon[(int)NOTIFY_ICON_POS.WEATHER].Icon!=null)
					array_NotifyIcon[(int)NOTIFY_ICON_POS.WEATHER].Icon.Dispose();

				System.IntPtr ptIco=bmp.GetHicon();

				array_NotifyIcon[(int)NOTIFY_ICON_POS.WEATHER].Icon=System.Drawing.Icon.FromHandle(ptIco);

				// Destroy the Icon, since the form creates
				// its own copy of the icon.
				DestroyIcon(ptIco);
                
				bmp.Dispose();
				// bmpCrop.Dispose();
				bitmap.Dispose();
			}


			if ((this.frm_weather_view.ShoudBeShow))
			{
				this.frm_weather_view.panel_wind.Visible=this.obj_options.show_wind_in_window;
				this.frm_weather_view.panel_pressure.Visible=this.obj_options.show_pressure_in_window;
				this.frm_weather_view.panel_weather.Visible=this.obj_options.show_clouds_in_window;
				this.frm_weather_view.panel_temp.Visible=this.obj_options.show_temp_in_window;
				this.frm_weather_view.Show();
				this.frm_weather_view.modify_display(this.obj_options.top_most,this.obj_options.opacity,this.obj_options.ColumnsPanelNumber);
			}

			// update details
			if (b_empty)
				this.frm_details.set_metar(null,this.obj_options);
			else
				this.frm_details.set_metar(obj_metar,this.obj_options);

		}
		public bool ThumbnailCallback()
		{
			return false;
		}
		public Bitmap GetThumb(Bitmap bmp,int width,int height)
		{
			Image.GetThumbnailImageAbort myCallback = new Image.GetThumbnailImageAbort(ThumbnailCallback);
			return (Bitmap)bmp.GetThumbnailImage(width, height, myCallback, IntPtr.Zero);
		}
		private Bitmap bitmap_crop(Bitmap b, int x_origin, int y_origin, int width, int height)
		{
			if ((x_origin+width>b.Width) || ( y_origin+height>b.Height) || (x_origin<0)|| (y_origin<0)|| (width<0)|| (height<0))
				throw new Exception("Invalid parameters");
			return b.Clone(new Rectangle(x_origin,y_origin,width,height),System.Drawing.Imaging.PixelFormat.Format32bppArgb);
		}

		private Bitmap bitmap_resize(Bitmap b, int x_origin, int y_origin, int width, int height)
		{
			Bitmap newImage = new Bitmap(width, height);
			System.Drawing.Graphics gr = System.Drawing.Graphics.FromImage(newImage);
			gr.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
			gr.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
			gr.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
			gr.DrawImage(b, new System.Drawing.Rectangle(0, 0, width, height));
			return newImage;
		}
        #endregion

        #region notify icons menu items
        private void menuItem_exit_Click(object sender, System.EventArgs e)
        {
            this.frm_weather_view.CloseForced();
            this.CloseForced();
        }

        private void menuItem_about_Click(object sender, System.EventArgs e)
        {
            this.frm_about.Text=this.obj_interface_language.about;
            this.frm_about.Show();
        }

        private void menuItem_details_Click(object sender, System.EventArgs e)
        {
            this.frm_details.Show();
        }

        private void menuItem_options_Click(object sender, System.EventArgs e)
        {
            this.show_me();
        }

        private void menuItem_hide_show_Click(object sender, System.EventArgs e)
        {
            if (this.frm_weather_view.Visible)
                this.frm_weather_view.Hide();
            else
                this.frm_weather_view.Show();
        }

        private void NotifyIcon_DoubleClick(object sender, EventArgs e)
        {
            this.menuItem_hide_show_Click(sender,e);
        }
        #endregion

        #region save options

        private bool save_options()
        {
            ClassOptions rollback_options=this.obj_options;
            string[] str_array=this.comboBox_cities.Text.Split(':');
            string city_name="";
            string country="";
            if (str_array.Length==2)
            {
                city_name=str_array[1].Trim();
                country=str_array[0].Trim();
            }
            this.obj_options.city=ClassCity.get_city_from_city_name_and_country(ref this.pcity_list,city_name,country);
            // if city code not found
            if (this.obj_options.city==null)
            {
                MessageBox.Show(this,"City not found in database","Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
                this.obj_options=rollback_options;
                this.show_me();
                return false;
            }

            this.obj_options.window_orientation=(ClassOptions.WINDOW_ORIENTATION)this.comboBox_orientation.SelectedIndex;
            this.obj_options.temp_unit=(ClassOptions.TEMP_UNITS)comboBox_temp_unit.SelectedIndex;
            this.obj_options.wind_unit=(ClassOptions.WIND_UNITS)this.comboBox_wind_unit.SelectedIndex;
            this.obj_options.pressure_unit=(ClassOptions.PRESSURE_UNITS)this.comboBox_pressure_unit.SelectedIndex;
            this.obj_options.length_unit=(ClassOptions.LENGTH_UNITS)this.comboBox_length_unit.SelectedIndex;
            this.obj_options.precipitation_height_unit=(ClassOptions.HEIGTH_UNITS)this.comboBox_precipitation_height_unit.SelectedIndex;
            this.obj_options.height_unit=(ClassOptions.LENGTH_UNITS)this.comboBox_height_unit.SelectedIndex;
            this.obj_options.show_clouds_in_satusbar=this.checkBox_clouds.Checked;
            this.obj_options.show_temp_in_satusbar=this.checkBox_temp.Checked;
            this.obj_options.show_wind_dir_in_satusbar=this.checkBox_wind_dir.Checked;
            this.obj_options.show_wind_speed_in_satusbar=this.checkBox_wind_speed.Checked;
            this.obj_options.show_clouds_in_window=this.checkBox_w_clouds.Checked;
            this.obj_options.show_temp_in_window=this.checkBox_w_temp.Checked;
            this.obj_options.show_wind_in_window=this.checkBox_w_wind.Checked;
            this.obj_options.show_pressure_in_window=this.checkBox_w_pressure.Checked;
            this.obj_options.top_most=this.checkBox_w_topmost.Checked;
            this.obj_options.opacity=((double)this.trackBar_w_opacity.Value)/100;
            this.obj_options.report_language=this.comboBox_language.Text;

            this.obj_options.proxy_used=this.checkBox_use_proxy.Checked;
            this.obj_options.proxy_ip=this.textBox_proxy_ip.Text;
            if (!ClassCheck_user_interface_inputs.check_ushort(this.textBox_proxy_port.Text))
            {
                this.obj_options=rollback_options;
                return false;
            }
            this.obj_options.proxy_port=System.Convert.ToUInt16(this.textBox_proxy_port.Text);
            this.obj_options.host=this.textBox_remote_host.Text;
            if (!ClassCheck_user_interface_inputs.check_ushort(this.textBox_remote_host_port.Text))
            {
                this.obj_options=rollback_options;
                return false;
            }
            this.obj_options.host_port=System.Convert.ToUInt16(this.textBox_remote_host_port.Text);
            this.obj_options.remote_dir=this.textBox_remote_host_directory.Text;
            if (!ClassCheck_user_interface_inputs.check_uint(this.textBox_update_interval.Text))
            {
                this.obj_options=rollback_options;
                return false;
            }
            this.obj_options.update_interval=System.Convert.ToUInt32(this.textBox_update_interval.Text);
            if (!ClassCheck_user_interface_inputs.check_uint(this.textBox_retry_interval.Text))
            {
                this.obj_options=rollback_options;
                return false;
            }
            this.obj_options.retry_interval=System.Convert.ToUInt32(this.textBox_retry_interval.Text);
            try
            {
                this.obj_options.proxy_type=(easy_socket.proxy.ClassProxy.PROXY_TYPE)(this.comboBox_proxy_type.SelectedIndex+1);
            }
            catch
            {
                this.obj_options.proxy_type=easy_socket.proxy.ClassProxy.PROXY_TYPE.HTTP_GET;
            }

            try
            {
                this.obj_options.parsing_type = (ClassMetarDownload.eparsing_type)(this.comboBox_reply_type.SelectedIndex);
            }
            catch
            {
                this.obj_options.parsing_type = ClassMetarDownload.eparsing_type.parsing_type_Text;
            }

            this.obj_options.notify_font_name = this.comboBox_notify_font_family.Text;
            this.obj_options.notify_font_size = this.numeric_UpDown_notify_font_size.Value;
            this.obj_options.notify_font_bold = this.checkBox_notify_font_bold.Checked;


            this.frm_details.set_language(this.exe_path+LANGUAGE_DIRECTORY+this.obj_options.report_language+".xml");
            this.set_notify_icon_text();

            this.obj_options.save(this.exe_path+CONFIG_FILENAME);

            return true;
        }

        #endregion

        private void set_notify_icon_text()
        {
            string str_wind_unit="";
            switch(this.obj_options.wind_unit)
            {
                case ClassOptions.WIND_UNITS.BEAUFORT:
                    str_wind_unit="B";
                    break;
                case ClassOptions.WIND_UNITS.MS:
                    str_wind_unit="Ms";
                    break;
                case ClassOptions.WIND_UNITS.MH:
                    str_wind_unit="Mh";
                    break;
                case ClassOptions.WIND_UNITS.KNOTS:
                    str_wind_unit="K";
                    break;
                case ClassOptions.WIND_UNITS.KMH:
                    str_wind_unit="Kmh";
                    break;
            }

            string str_temp_unit="";
            switch(this.obj_options.temp_unit)
            {
                case ClassOptions.TEMP_UNITS.CELCIUS:
                    str_temp_unit="C";
                    break;
                case ClassOptions.TEMP_UNITS.FAHRENHEIT:
                    str_temp_unit="F";
                    break;
            }

            this.array_NotifyIcon[(int)NOTIFY_ICON_POS.WEATHER].Text =this.obj_interface_language.weather;
            this.array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_IMG].Text = this.obj_interface_language.wind_direction;
            this.array_NotifyIcon[(int)NOTIFY_ICON_POS.TEMP].Text = this.obj_interface_language.temperature+" ("+str_temp_unit+")";
            this.array_NotifyIcon[(int)NOTIFY_ICON_POS.WIND_DATA].Text = this.obj_interface_language.wind_speed+" ("+str_wind_unit+")";
        }
        public void show_me()
        {
            this.ShowInTaskbar=true;
            this.WindowState=FormWindowState.Normal;
            this.Show();
            this.Activate();
        }

        public void start()
        {
            if (this.obj_options.city.code!="")
                this.start_from_config();
            else
                this.show_me();
        }

        private void start_from_config()
        {
           this.obj_metar_download.start(this.obj_options.city.code,
               this.obj_options.update_interval,
               this.obj_options.retry_interval,
               this.obj_options.host,
               this.obj_options.host_port,
               this.obj_options.remote_dir,
               this.obj_options.proxy_used,
               this.obj_options.proxy_ip,
               this.obj_options.proxy_port,
               this.obj_options.proxy_type,
               this.obj_options.parsing_type);

           #region notify icon showing
            int[] array =new int[this.notify_icon_number];
            for (int cnt=0;cnt<this.notify_icon_number;cnt++)
                array[cnt]=-1;
            int nb_visible_icon=0;
            if (this.obj_options.show_clouds_in_satusbar)
            {
                array[(int)NOTIFY_ICON_POS.WEATHER]=(int)NOTIFY_ICON_POS.WEATHER;
                nb_visible_icon++;
            }
            if (this.obj_options.show_temp_in_satusbar)
            {
                array[(int)NOTIFY_ICON_POS.TEMP]=(int)NOTIFY_ICON_POS.TEMP;
                nb_visible_icon++;
            }
            if (this.obj_options.show_wind_dir_in_satusbar)
            {
                array[(int)NOTIFY_ICON_POS.WIND_IMG]=(int)NOTIFY_ICON_POS.WIND_IMG;
                nb_visible_icon++;
            }
            if (this.obj_options.show_wind_speed_in_satusbar)
            {
                array[(int)NOTIFY_ICON_POS.WIND_DATA]=(int)NOTIFY_ICON_POS.WIND_DATA;
                nb_visible_icon++;
            }
            if (nb_visible_icon>0)
            {
                int[] array_visible=new int[nb_visible_icon];
                int cnt2=0;
                for (int cnt=0;cnt<this.notify_icon_number;cnt++)
                {
                    if (array[cnt]!=-1)
                    {
                        array_visible[cnt2]=array[cnt];
                        cnt2++;
                    }
                }
                // show wanted notify icons only
                this.show_ordered_notify_icons(array_visible);
            }
            #endregion

        }

        #region form events

        public bool hide_query()
        {
            return this.hide_query(false);
        }
        public bool hide_query(bool check_only)
        {
            // if this is the only shown form close it
            bool b_no_icon=!(this.obj_options.show_clouds_in_satusbar||this.obj_options.show_temp_in_satusbar
                ||this.obj_options.show_wind_dir_in_satusbar||this.obj_options.show_wind_speed_in_satusbar
                );
            if ((!this.frm_weather_view.ShoudBeShow)&&b_no_icon)
                return false;
            b_no_icon=true;
            for (int cnt=0;cnt<this.notify_icon_number;cnt++)
            {
                if ((this.array_NotifyIcon[cnt].Visible==true)&&(this.array_NotifyIcon[cnt].Icon!=null))
                {
                    b_no_icon=false;
                    break;
                }
            }
//            if (b_no_icon&&(!this.obj_metar_download.Started)&&(!this.frm_weather_view.Visible))
            if (b_no_icon&&!this.frm_weather_view.Visible)
                return false;

            if (!check_only)
                this.Hide();
            return true;
        }

        public void CloseForced()
        {
            this.b_closed_forced=true;
            this.Close();
        }
        private void FormMain_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (this.b_closed_forced)
            {
                this.obj_metar_download.stop();
                return;
            }
            this.hide_query();
            // else hide
            e.Cancel=true;
        }
        private const int WM_QUERYENDSESSION=0x0011;
        protected override void WndProc(ref Message uMsg)
        {
            switch(uMsg.Msg)
            {
                case WM_QUERYENDSESSION:
                    this.b_closed_forced=true;
                    break;
            }
            base.WndProc(ref uMsg);
        }
        private void button_exit_Click(object sender, System.EventArgs e)
        {
            this.CloseForced();
        }
        private void button_cancel_Click(object sender, System.EventArgs e)
        {
            this.hide_query();
        }

        private void button_apply_Click(object sender, System.EventArgs e)
        {
            if (this.save_options())
            {
                this.update_language_interface();
                // update visual information
                this.update_information(this.last_raw_metar);
                this.start_from_config();
            }
            this.hide_query();       
        }

        private void show_state()
        {
            // update current state
            if (this.obj_metar_download.Started)
            {
                this.label_current_state.Text=this.obj_interface_language.started+"\r\n"
                    +this.obj_interface_language.city+": "
                    +this.obj_options.city.name;
            }
            else
            {
                this.label_current_state.Text=this.obj_interface_language.stopped;
            }
        }
        private void FormMain_Activated(object sender, System.EventArgs e)
        {
            this.show_state();
        }

        private void FormMain_Load(object sender, System.EventArgs e)
        {
            this.start();
        }

        private void checkBox_use_proxy_CheckedChanged(object sender, System.EventArgs e)
        {
            this.groupBox_proxy.Enabled=this.checkBox_use_proxy.Checked;
        }
        #endregion

        private void update_language_interface()
        {
			int SelectedIndex;

            this.obj_interface_language=ClassInterfaceLanguage.load(this.exe_path+LANGUAGE_INTERFACE_DIRECTORY+this.obj_options.report_language+".xml");

            this.Text=this.obj_interface_language.options;

			//////////////////////////////////////////////////////////////
			/// tabs title
			//////////////////////////////////////////////////////////////
            this.tabControl_main.TabPages[0].Text=this.obj_interface_language.general;
            this.tabControl_main.TabPages[1].Text=this.obj_interface_language.units;
            this.tabControl_main.TabPages[2].Text=this.obj_interface_language.connection;

			//////////////////////////////////////////////////////////////
			/// main tab
			//////////////////////////////////////////////////////////////
            this.groupBox_current_state.Text=this.obj_interface_language.current_state;
            this.groupBox_language.Text=this.obj_interface_language.language;
            this.groupBox_Window.Text=this.obj_interface_language.window;
			this.label_city.Text=this.obj_interface_language.country_city;
			this.label_orientation.Text=this.obj_interface_language.orientation;

			this.checkBox_clouds.Text=this.obj_interface_language.weather;
            this.checkBox_temp.Text=this.obj_interface_language.temperature;
            this.checkBox_w_clouds.Text=this.obj_interface_language.weather;
            this.checkBox_w_pressure.Text=this.obj_interface_language.pressure;
            this.checkBox_w_temp.Text=this.obj_interface_language.temperature;
            this.checkBox_w_topmost.Text=this.obj_interface_language.top_most_window;
            this.checkBox_w_wind.Text=this.obj_interface_language.wind;
            this.checkBox_wind_dir.Text=this.obj_interface_language.wind_direction;
            this.checkBox_wind_speed.Text=this.obj_interface_language.wind_speed;

			this.label_window_opacity.Text=this.obj_interface_language.window_opacity;
			// save selection
			SelectedIndex = this.comboBox_orientation.SelectedIndex;
			this.comboBox_orientation.Items.Clear();
			this.comboBox_orientation.Items.AddRange(new object[] {this.obj_interface_language.horizontal,
																	  this.obj_interface_language.vertical
																  });
			// restore selection
			this.comboBox_orientation.SelectedIndex=SelectedIndex;

			this.groupBoxnotify_font.Text = this.obj_interface_language.notify_font;
			this.label_notify_font_family.Text = this.obj_interface_language.family;
			this.label_notify_font_size.Text = this.obj_interface_language.size;
			this.checkBox_notify_font_bold.Text = this.obj_interface_language.bold;


			//////////////////////////////////////////////////////////////
			/// Unit tab
			//////////////////////////////////////////////////////////////
			this.label_temp_unit.Text=this.obj_interface_language.temperature;
			SelectedIndex = this.comboBox_temp_unit.SelectedIndex;
			this.comboBox_temp_unit.Items.Clear();
			this.comboBox_temp_unit.Items.AddRange(new object[] {this.obj_interface_language.fahrenheit,
																 this.obj_interface_language.celsius
																});
			this.comboBox_temp_unit.SelectedIndex=SelectedIndex;

			this.label_wind_unit.Text=this.obj_interface_language.wind;
			SelectedIndex = this.comboBox_wind_unit.SelectedIndex;
			this.comboBox_wind_unit.Items.Clear();
			this.comboBox_wind_unit.Items.AddRange(new object[] {this.obj_interface_language.meters_per_second,
																this.obj_interface_language.kilometers_per_hour,
																this.obj_interface_language.miles_per_hour,
																this.obj_interface_language.knots,
																this.obj_interface_language.beaufort
																});
			this.comboBox_wind_unit.SelectedIndex=SelectedIndex;

			this.label_pressure_unit.Text=this.obj_interface_language.pressure;

			this.label_length_unit.Text=this.obj_interface_language.length;
			SelectedIndex = this.comboBox_length_unit.SelectedIndex;
			this.comboBox_length_unit.Items.Clear();
			this.comboBox_length_unit.Items.AddRange(new object[] {this.obj_interface_language.feet,
																this.obj_interface_language.kilometers,
																this.obj_interface_language.meters,
																this.obj_interface_language.miles,
																});
			this.comboBox_length_unit.SelectedIndex=SelectedIndex;

			this.label_precipitation_height_unit.Text=this.obj_interface_language.precipitation_height;
			SelectedIndex = this.comboBox_precipitation_height_unit.SelectedIndex;
			this.comboBox_precipitation_height_unit.Items.Clear();
			this.comboBox_precipitation_height_unit.Items.AddRange(new object[] {this.obj_interface_language.inch,
																				this.obj_interface_language.millimeters
																				});
			this.comboBox_precipitation_height_unit.SelectedIndex=SelectedIndex;

			this.label_height_unit.Text=this.obj_interface_language.height; 
			SelectedIndex = this.comboBox_height_unit.SelectedIndex;
			this.comboBox_height_unit.Items.Clear();
			this.comboBox_height_unit.Items.AddRange(new object[] {this.obj_interface_language.feet,
																	  this.obj_interface_language.kilometers,
																	  this.obj_interface_language.meters,
																	  this.obj_interface_language.miles,
																	});
			this.comboBox_height_unit.SelectedIndex=SelectedIndex;

			//////////////////////////////////////////////////////////////
			/// Connection tab
			//////////////////////////////////////////////////////////////
            this.label_proxy_ip.Text=this.obj_interface_language.proxy_ip;
            this.label_proxy_port.Text=this.obj_interface_language.proxy_port;
            this.label_proxy_type.Text=this.obj_interface_language.proxy_type;
            this.label_remote_host_directory.Text=this.obj_interface_language.remote_host_directory;
            this.label_remote_host_ip.Text=this.obj_interface_language.remote_host_ip;
            this.label_remote_host_port.Text=this.obj_interface_language.remote_host_port;
            this.checkBox_use_proxy.Text=this.obj_interface_language.use_proxy;
            this.label_update_interval.Text=this.obj_interface_language.update_interval;
            this.label_retry_interval.Text=this.obj_interface_language.retry_interval;
			this.label_received_data_type.Text = this.obj_interface_language.received_data_type;

			// save selection
			SelectedIndex = this.comboBox_reply_type.SelectedIndex;
			this.comboBox_reply_type.Items.Clear();
			this.comboBox_reply_type.Items.AddRange(new object[] {this.obj_interface_language.received_data_type_text,
																	 "XML"
																 });
			// restore selection
			this.comboBox_reply_type.SelectedIndex=SelectedIndex;

			//////////////////////////////////////////////////////////////
			/// apply cancel exit
			//////////////////////////////////////////////////////////////
            this.button_apply.Text=this.obj_interface_language.apply;
            this.button_cancel.Text=this.obj_interface_language.cancel;
            this.button_exit.Text=this.obj_interface_language.exit;

			//////////////////////////////////////////////////////////////
			/// popup menu
			//////////////////////////////////////////////////////////////
            this.menuItem_about.Text=this.obj_interface_language.about;
            this.menuItem_details.Text=this.obj_interface_language.details;
            this.menuItem_exit.Text=this.obj_interface_language.exit;
            this.menuItem_hide_show.Text=this.obj_interface_language.show_hide;
            this.menuItem_options.Text=this.obj_interface_language.options;


            this.show_state();

            this.frm_details.Text=this.obj_interface_language.details;

            this.frm_weather_view.set_tooltips_text(this.obj_interface_language.details,this.obj_interface_language.options);

        }


    }

}
